@@ -59,11 +59,7 @@ class Application(object):
     def __init__(self, property_window):
         self.property_window = property_window
         self.notebook = wx.Notebook(self.property_window, -1)
-        if not misc.check_wx_version(2, 5, 2):
-            nb_sizer = wx.NotebookSizer(self.notebook)
-            self.notebook.sizer = nb_sizer
-        else:
-            self.notebook.sizer = None
+        self.notebook.sizer = None
         self.notebook.SetAutoLayout(True)
         self.notebook.Hide()
         panel = wx.ScrolledWindow(
@@ -172,7 +168,7 @@ class Application(object):
         self.codewriters_prop.set_str_value('python')
 
         self.for_version_prop = RadioProperty(self, "for_version", panel,
-                                              ['2.4', '2.6', '2.8'], columns=3,
+                                              ['2.6', '2.8'], columns=2,
                                               label=_("wxWidgets compatibility"))
         self.for_version_prop.set_str_value(self.for_version)
         
