@@ -1,7 +1,7 @@
 # lisp_codegen.py: lisp code generator
-# $Id: lisp_codegen.py,v 1.5 2006/11/30 16:25:33 jkt Exp $
 #
 # Copyright (c) 2005 Surendra K Singhi <efuzzyone@users.sourceforge.net>
+#
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
@@ -18,54 +18,80 @@ the lines to add to the '__init__', '__set_properties' and '__do_layout'
 methods of the parent object.
 """
 
-import sys, os, os.path
-import common
+import os
+import os.path
+import re
+import sys
+
+import common, config
 import cStringIO
 from xml_parse import XmlParsingError
-import re
+
+try:
+    set
+except NameError:
+    from sets import Set as set
 
 
 # these two globals must be defined for every code generator module
 language = 'lisp'
-writer = sys.modules[__name__] # the writer is the module itself
-
-# default extensions for generated files: a list of file extensions
-default_extensions = ['lisp']
-
+writer = sys.modules[__name__]
 """\
-dictionary that maps the lines of code of a class to the name of such class:
+The writer is the module itself
+"""
+
+default_extensions = ['lisp']
+"""\
+Default extensions for generated files: a list of file extensions
+"""
+
+classes = None
+"""\
+Dictionary that maps the lines of code of a class to the name of such class:
 the lines are divided in 3 categories: '__init__', '__set_properties' and
 '__do_layout'
 """
-classes = None
 
-"""dictionary of ``writers'' for the various objects"""
 obj_builders = {}
-
 """\
-dictionary of ``property writer'' functions, used to set the properties of a
+Dictionary of ``writers'' for the various objects
+"""
+
+obj_properties = {}
+"""\
+Dictionary of ``property writer'' functions, used to set the properties of a
 toplevel object
 """
-obj_properties = {}
 
-# random number used to be sure that the replaced tags in the sources are
-# the right ones (see SourceFileContent and add_class)
 nonce = None
+"""\
+Random number used to be sure that the replaced tags in the sources are
+the right ones (see SourceFileContent and add_class)
+"""
 
-# lines common to all the generated files (import of wxCL, ...)
 header_lines = []
+"""\
+Lines common to all the generated files (import of wxCL, ...)
+"""
 
 class_lines = []
 
 init_lines = []
 
-# if True, generate a file for each custom class
 multiple_files = False
+"""\
+If True, generate a file for each custom class
+"""
 
-# if not None, it is the single source file to write into
 output_file = None
-# if not None, it is the directory inside which the output files are saved
+"""\
+If not None, it is the single source file to write into
+"""
+
 out_dir = None
+"""\
+If not None, it is the directory inside which the output files are saved
+"""
 
 import_packages = []
 
@@ -86,9 +112,8 @@ def cn_f(flags):
         return str(flags)
 
 
-# ALB 2004-12-05: wx version we are generating code for
 for_version = (2, 4)
-
+"""wx version we are generating code for"""
 
 class ClassLines:
     """\
@@ -258,24 +283,31 @@ class SourceFileContent:
 
 # end of class SourceFileContent
 
-# if not None, it is an instance of SourceFileContent that keeps info about
-# the previous version of the source to generate
 previous_source = None 
-
+"""\
+If not None, it is an instance of SourceFileContent that keeps info about
+the previous version of the source to generate
+"""
 
 def tabs(number):
+    """\
+    Returns spaces to indent given number of levels
+    """
     return '    ' * number
 
-
-# if True, overwrite any previous version of the source file instead of
-# updating only the wxGlade blocks
 _overwrite = False
+"""\
+If True, overwrite any previous version of the source file instead of
+updating only the wxGlade blocks
+"""
 
-# if True, enable gettext support
 _use_gettext = False
-
+"""\
+If True, enable gettext support
+"""
 
 _quote_str_pattern = re.compile(r'\\[natbv"]?')
+
 def _do_replace(match):
     if match.group(0) == '\\': return '\\\\'
     else: return match.group(0)
@@ -297,15 +329,22 @@ def quote_str(s, translate=True, escape_chars=True):
         if _use_gettext and translate: return '_(u"' + s + '")'
         else: return 'u"' + s + '"'
 
+def quote_path(s):
+    """\
+    escapes all " and \ , thus making a path suitable to
+    insert in a list source file
+    """ # " alb: comment needed to avoid emacs going insane with colorization..
+    s = s.replace('\\', '\\\\')
+    s = s.replace('"', r'\"')
+    return '"' + s + '"'
 
 def initialize(app_attrs): 
     """\
     Writer initialization function.
-    - app_attrs: dict of attributes of the application. The following two
-        are always present:
-            path: output path for the generated code (a file if multi_files is
-                False, a dir otherwise)
-            option: if True, generate a separate file for each custom class
+
+    @keyword path: Output path for the generated code (a file if multi_files is
+                   False, a dir otherwise)
+    @keyword option: If True, generate a separate file for each custom class
     """
     out_path = app_attrs['path']
     multi_files = app_attrs['option']
@@ -341,35 +380,6 @@ def initialize(app_attrs):
         else:
             for_version = (2, 4) # default...
 
-# scan widgets.txt for widgets, load lisp_codegen's
-    _widgets_dir = os.path.join(common.wxglade_path, 'widgets')
-    widgets_file = os.path.join(_widgets_dir, 'widgets.txt')
-    if not os.path.isfile(widgets_file):
-        print >> sys.stderr, "widgets file (%s) doesn't exist" % widgets_file
-        return
-
-    import sys
-    sys.path.append(_widgets_dir)
-    modules = open(widgets_file)
-    for line in modules:
-        module_name = line.strip()
-        if not module_name or module_name.startswith('#'): continue
-        module_name = module_name.split('#')[0].strip()
-        try:
-            m = __import__(module_name + '.lisp_codegen', {}, {}, ['initialize'])
-            m.initialize()
-        except (ImportError, AttributeError):
-            print 'ERROR loading "%s"' % module_name
-            import traceback;
-            traceback.print_exc()
-#        else:
-#            print 'initialized lisp generator for ', module_name
-    modules.close()
-
-   # ...then, the sizers
-    import edit_sizers.lisp_sizers_codegen
-    edit_sizers.lisp_sizers_codegen.initialize()  
-
     try:
         _use_gettext = int(app_attrs['use_gettext'])
     except (KeyError, ValueError):
@@ -388,9 +398,15 @@ def initialize(app_attrs):
     header_lines = [';;; generated by wxGlade %s on %s%s\n\n' % \
                     (common.version, time.asctime(), common.generated_from()),
                     '(asdf:operate \'asdf:load-op \'wxcl)\n',
-                    '(use-package \"FFI\")\n(ffi:default-foreign-language :stdc)\n\n'
+                    '(use-package \"FFI\")\n'
+                    '(ffi:default-foreign-language :stdc)\n\n'
                     ]
-    import_packages = set (["wxCL","wxFrame","wx_main","wx_wrapper","wxWindow","wxColour","wxEvtHandler","wxEvent"])
+    if not config.preferences.write_timestamp:
+        header_lines[0] = ';;; generated by wxGlade %s%s\n\n' % \
+                          (common.version, common.generated_from())
+
+    import_packages = set (["wxCL", "wxFrame", "wx_main", "wx_wrapper",
+                            "wxWindow", "wxColour", "wxEvtHandler", "wxEvent"])
 # add coding (PEP 263)
 #    if _encoding:
 #       header_lines.insert(0, "# -*- coding: %s -*-\n" % _encoding)
@@ -567,14 +583,17 @@ def add_sizeritem(toplevel, sizer, obj, option, flag, border):
     
     sizer.name = sizer.name.replace('_','-')
     obj_name = obj.name
-    try: w, h = [ int(s) for s in obj_name.split(',') ]
+    try:
+        w, h = [ int(s) for s in obj_name.split(',') ]
     except ValueError:
         if obj.base == 'wxNotebook' and for_version < (2, 5):
-              obj_name = cn('wxNotebookSizer') + '(%s)' % obj_name
+            obj_name = cn('wxNotebookSizer') + '(%s)' % obj_name
     else:
         obj_name = '(%d, %d)' % (w, h) # it was the dimension of a spacer
-    try: klass = classes[toplevel.klass]
-    except KeyError: klass = classes[toplevel.klass] = ClassLines()
+    try:
+        klass = classes[toplevel.klass]
+    except KeyError:
+        klass = classes[toplevel.klass] = ClassLines()
 
     flag = '%s' % cn_f(flag)
     flag = flag.strip().replace('|',' ')
@@ -588,7 +607,6 @@ def add_sizeritem(toplevel, sizer, obj, option, flag, border):
     else:
         buffer = '(wxSizer_AddWindow (slot-%s obj) (slot-%s obj) %s %s %s nil)\n' % \
                  (sizer.name, obj_name, option, flag,cn_f(border))
-    print 
     klass.layout.append(buffer)
 
 
@@ -605,7 +623,7 @@ def add_class(code_obj):
         # let's see if the file to generate exists, and in this case
         # create a SourceFileContent instance
         filename = os.path.join(out_dir,
-                                code_obj.klass.replace('.', '_') + '.py')
+                                code_obj.klass.replace('.', '_') + '.lisp')
         if _overwrite or not os.path.exists(filename): prev_src = None
         else: prev_src = SourceFileContent(filename)
         _current_extra_modules = {}
@@ -661,8 +679,10 @@ def add_class(code_obj):
         else:
             klass = code_obj.klass
         write('\n(defclass %s()\n' % klass)
-        write("\t((top-window :initform nil :accessor slot-top-window)")
-        for l in class_lines: write("\n"+tab+"("+l+" :initform nil :accessor slot-"+l+")")
+        write(tab + "((top-window :initform nil :accessor slot-top-window)")
+        for l in class_lines:
+            write("\n")
+            write(tab+"("+l+" :initform nil :accessor slot-"+l+")")
         write("))\n")
 
         write("\n(defun make-%s ()\n" % klass)
@@ -724,11 +744,11 @@ def add_class(code_obj):
         for win_id, event, handler in event_handlers:
             if win_id.startswith('#'):
                 write(tab + "(wxEvtHandler_Connect (slot-top-window obj) %s (exp%s)"
-                      "\n\t\t(wxClosure_Create #'%s obj))\n"
+                      "\n" + tabs(2) + "(wxClosure_Create #'%s obj))\n"
                       % (win_id[1:],event, handler, ))
             else:
                 write(tab + "(wxEvtHandler_Connect (slot-top-window obj) %s (exp%s)"
-                      "\n\t\t(wxClosure_Create #'%s obj))\n"
+                      "\n" + tabs(2) + "(wxClosure_Create #'%s obj))\n"
                       % (win_id,event, handler, ))
 
     # end tag
@@ -883,12 +903,12 @@ def add_class(code_obj):
                 # store the new file contents to disk
                 common.save_file(filename, prev_src.content, 'codegen')
             except:
-                raise IOError("py_codegen.add_class: %s, %s, %s" % \
+                raise IOError("lisp_codegen.add_class: %s, %s, %s" % \
                               (out_dir, prev_src.name, code_obj.klass))
             return
 
         # create the new source file
-        filename = os.path.join(out_dir, code_obj.klass + '.py')
+        filename = os.path.join(out_dir, code_obj.klass + '.lisp')
         out = cStringIO.StringIO()
         write = out.write
         # write the common lines
@@ -932,7 +952,7 @@ def add_app(app_attrs, top_win_class):
 
     if not multiple_files: prev_src = previous_source
     else:
-        filename = os.path.join(out_dir, name + '.py')
+        filename = os.path.join(out_dir, name + '.lisp')
         if not os.path.exists(filename): prev_src = None
         else:
             # prev_src doesn't need to be a SourceFileContent instance in this
@@ -980,10 +1000,11 @@ def add_app(app_attrs, top_win_class):
         append(tab + '(ELJApp_SetTopWindow (slot-top-window %s))\n' % top_win)
         append(tab + '(wxWindow_Show (slot-top-window %s))))\n' % top_win)
 
-    append("\n(unwind-protect\n\t(Eljapp_initializeC (wxclosure_Create #'init-func nil) 0 nil)")
-    append("\n  (ffi:close-foreign-library \"../miscellaneous/wxc-msw2.6.2.dll\"))\n")
+    append("\n(unwind-protect\n")
+    append(tab + "(Eljapp_initializeC (wxclosure_Create #'init-func nil) 0 nil)\n")
+    append(tab + "(ffi:close-foreign-library \"../miscellaneous/wxc-msw2.6.2.dll\"))\n")
     if multiple_files:
-        filename = os.path.join(out_dir, name + '.py')
+        filename = os.path.join(out_dir, name + '.lisp')
         out = cStringIO.StringIO()
         write = out.write
         write('#!/usr/bin/env lisp\n')
@@ -1049,7 +1070,7 @@ def generate_code_foreground(obj):
         color = cn('(wxSystemSettings_GetColour ') + '%s)' % \
                 cn(obj.properties['foreground'])
     import_packages = import_packages | set(['wxColour'])
-    return self + '(wxWindow_SetForegroundColour %s %s)\n' % (self,color)
+    return self + '(wxWindow_SetForegroundColour %s %s)\n' % (self, color)
 
 
 def generate_code_background(obj):
@@ -1067,7 +1088,7 @@ def generate_code_background(obj):
                 cn(obj.properties['background'])
     import_packages = import_packages | set(['wxColour'])
     print import_packages
-    return  '(wxWindow_SetBackgroundColour %s %s)\n' % (self,color)
+    return  '(wxWindow_SetBackgroundColour %s %s)\n' % (self, color)
 
 
 def generate_code_font(obj):
@@ -1247,22 +1268,29 @@ class EventsPropertyHandler(object):
 # end of class EventsPropertyHandler
     
 
-# dictionary whose items are custom handlers for widget properties
 _global_property_writers = { 'font': FontPropertyHandler,
                              'events': EventsPropertyHandler, }
+"""\
+Dictionary whose items are custom handlers for widget properties
+"""
 
-# dictionary of dictionaries of property handlers specific for a widget
-# the keys are the class names of the widgets
-# Ex: _property_writers['wxRadioBox'] = {'choices', choices_handler}
 _property_writers = {}
+"""\
+Dictionary of dictionaries of property handlers specific for a widget
+the keys are the class names of the widgets
+Ex: _property_writers['wxRadioBox'] = {'choices', choices_handler}
+"""
 
-
-# map of widget class names to a list of extra modules needed for the
-# widget. Example: 'wxGrid': 'from wxLisp.grid import *\n'
 _widget_extra_modules = {}
+"""\
+Map of widget class names to a list of extra modules needed for the
+widget. Example: 'wxGrid': 'from wxLisp.grid import *\n'
+"""
 
-# set of lines of extra modules to add to the current file
 _current_extra_modules = {} 
+"""\
+Set of lines of extra modules to add to the current file
+"""
 
 def get_property_handler(property_name, widget_name):
     try: cls = _property_writers[widget_name][property_name]
@@ -1272,7 +1300,7 @@ def get_property_handler(property_name, widget_name):
 
 def add_property_handler(property_name, handler, widget_name=None):
     """\
-    sets a function to parse a portion of XML to get the value of the property
+    Sets a function to parse a portion of XML to get the value of the property
     property_name. If widget_name is not None, the function is called only if
     the property in inside a widget whose class is widget_name
     """
@@ -1288,8 +1316,8 @@ class WidgetHandler:
     Interface the various code generators for the widgets must implement
     """
     
-    """list of modules to import (eg. ['from wxLisp.grid import *\n'])"""
     import_modules = []
+    """list of modules to import (eg. ['from wxLisp.grid import *\n'])"""
 
     def get_code(self, obj):
         """\
@@ -1330,3 +1358,38 @@ class WidgetHandler:
 def add_widget_handler(widget_name, handler):
     obj_builders[widget_name] = handler
 
+
+def setup():
+    """\
+    Code generator setup function. This is called once, when the code
+    generator is loaded in wxglade.
+    """
+    # scan widgets.txt for widgets, load lisp_codegen's
+    widgets_file = os.path.join(common.widgets_path, 'widgets.txt')
+    if not os.path.isfile(widgets_file):
+        print >> sys.stderr, "widgets file (%s) doesn't exist" % widgets_file
+        return
+
+    sys.path.append(common.widgets_path)
+    modules = open(widgets_file)
+    for line in modules:
+        module_name = line.strip()
+        if not module_name or module_name.startswith('#'): continue
+        module_name = module_name.split('#')[0].strip()
+        try:
+            m = __import__(module_name + '.lisp_codegen', {}, {},
+                           ['initialize'])
+            m.initialize()
+        except (ImportError, AttributeError):
+            pass
+##             print 'ERROR loading "%s"' % module_name
+##             import traceback;
+##             traceback.print_exc()
+##         else:
+##             print 'initialized lisp generator for ', module_name
+    modules.close()
+
+    # ...then, the sizers
+    import edit_sizers.lisp_sizers_codegen
+    edit_sizers.lisp_sizers_codegen.initialize()  
+    
