@@ -1,5 +1,5 @@
 # py_codegen.py: python code generator
-# $Id: py_codegen.py,v 1.65 2007/03/27 07:02:06 agriggio Exp $
+# $Id: py_codegen.py,v 1.66 2007/08/07 12:15:21 agriggio Exp $
 #
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
@@ -20,50 +20,57 @@ methods of the parent object.
 """
 
 import sys, os, os.path
-import common
+import common, config
 import cStringIO
 from xml_parse import XmlParsingError
 import re
 
 
-# these two globals must be defined for every code generator module
 language = 'python'
-writer = sys.modules[__name__] # the writer is the module itself
+"""These two globals must be defined for every code generator module"""
+
+writer = sys.modules[__name__]
+"""The writer is the module itself"""
 
-# default extensions for generated files: a list of file extensions
 default_extensions = ['py']
+"""Default extensions for generated files: a list of file extensions"""
 
+classes = None
 """\
 dictionary that maps the lines of code of a class to the name of such class:
 the lines are divided in 3 categories: '__init__', '__set_properties' and
 '__do_layout'
 """
-classes = None
 
-"""dictionary of ``writers'' for the various objects"""
 obj_builders = {}
+"""dictionary of ``writers'' for the various objects"""
 
+obj_properties = {}
 """\
 dictionary of ``property writer'' functions, used to set the properties of a
 toplevel object
 """
-obj_properties = {}
 
-# random number used to be sure that the replaced tags in the sources are
-# the right ones (see SourceFileContent and add_class)
 nonce = None
+"""\
+Random number used to be sure that the replaced tags in the sources are
+the right ones (see SourceFileContent and add_class)
+"""
 
-# lines common to all the generated files (import of wxPython, ...)
 header_lines = []
+"""Lines common to all the generated files (import of wxPython, ...)"""
 
-# if True, generate a file for each custom class
 multiple_files = False
+"""If True, generate a file for each custom class"""
 
 # if not None, it is the single source file to write into
 output_file = None
-# if not None, it is the directory inside which the output files are saved
-out_dir = None
+output_file_name = None
 
+out_dir = None
+"""\
+If not None, it is the directory inside which the output files are saved
+"""
 
 # ALB 2003-11-20: transition to the new wxPython, i.e. from:
 # from wxPython.wx import *
@@ -75,8 +82,16 @@ out_dir = None
 # w = wx.Window(...)
 
 use_new_namespace = True
+"""If True use the new name space (import wx)"""
+
+_app_added = False
+"""True after wxApp instance has been generated"""
 
 def cn(class_name):
+    """\
+    Return the class name properly formatted for the selected name space.
+    @see: L{use_new_namespace}
+    """
     if use_new_namespace:
         if class_name[:2] == 'wx': 
             return 'wx.' + class_name[2:]
@@ -85,35 +100,50 @@ def cn(class_name):
     return class_name
 
 def cn_f(flags):
+    """\
+    Return the flags properly formatted for the selected name space.
+    @see: L{use_new_namespace}
+    """
     if use_new_namespace:
-        return "|".join([cn(f) for f in str(flags).split('|')])
+        return " | ".join([cn(f) for f in str(flags).split('|')])
     else:
         return str(flags)
 
 
-# ALB 2004-12-05: wx version we are generating code for
 for_version = (2, 4)
+"""wx version we are generating code for"""
 
 
 class ClassLines:
     """\
     Stores the lines of python code for a custom class
+
+    @ivar dependencies:      Names of the modules this class depends on
+    @ivar event_handlers:    Lines to bind events
+    @ivar extra_code:        Extra code to output before this class
+    @ivar done:              If True, the code for this class has already
+                             been generated
+    @ivar init:              Lines of code to insert in the __init__ method
+                             (for children widgets)
+    @ivar layout:            Lines to insert in the __do_layout method
+    @ivar parents_init:      Lines of code to insert in the __init__ for
+                             container widgets (panels, splitters, ...)
+    @ivar props:             Lines to insert in the __set_properties method
+    @ivar self.sizers_init : Lines related to sizer objects declarations
     """
     def __init__(self):
-        self.init = [] # lines of code to insert in the __init__ method
-                       # (for children widgets)
-        self.parents_init = [] # lines of code to insert in the __init__ for
-                               # container widgets (panels, splitters, ...)
-        self.sizers_init = [] # lines related to sizer objects declarations
-        self.props = [] # lines to insert in the __set_properties method
-        self.layout = [] # lines to insert in the __do_layout method
-        
-        self.dependencies = {} # names of the modules this class depends on
-        self.done = False # if True, the code for this class has already
-                          # been generated
-
-        # ALB 2004-12-05
-        self.event_handlers = [] # lines to bind events 
+        self.deps = []
+        self.child_order = []
+        self.init_lines = {}
+        self.init = [] 
+        self.parents_init = []
+        self.sizers_init = []
+        self.props = []
+        self.layout = []
+        self.dependencies = {}
+        self.done = False
+        self.event_handlers = []
+        self.extra_code = []
 
 # end of class ClassLines
 
@@ -122,20 +152,25 @@ class SourceFileContent:
     """\
     Keeps info about an existing file that has to be updated, to replace only
     the lines inside a wxGlade block, an to keep the rest of the file as it was
+   
+    @ivar classes:        Classes declared in the file
+    @ivar content:        Content of the source file, if it existed
+                          before this session of code generation
+    @ivar event_handlers: List of event handlers for each class
+    @ivar name:           Name of the file
+    @ivar new_classes:    New classes to add to the file (they are inserted
+                          BEFORE the old ones)
+    @ivar spaces:         Indentation level for each class
     """
     def __init__(self, name=None, content=None, classes=None):
-        self.name = name # name of the file
-        self.content = content # content of the source file, if it existed
-                               # before this session of code generation
-        self.classes = classes # classes declared in the file
-        self.new_classes = [] # new classes to add to the file (they are
-                              # inserted BEFORE the old ones)
-        if classes is None: self.classes = {}
-        self.spaces = {} # indentation level for each class
-
-        # ALB 2004-12-05
-        self.event_handlers = {} # list of event handlers for each class
-
+        self.name = name
+        self.content = content
+        self.classes = classes
+        self.new_classes = []
+        if classes is None:
+            self.classes = {}
+        self.spaces = {}
+        self.event_handlers = {}
         if self.content is None:
             self.build_untouched_content()
 
@@ -190,7 +225,8 @@ class SourceFileContent:
                     inside_triple_quote = False
             
             result = class_decl.match(line)
-            if not inside_triple_quote and result is not None:
+            if not inside_triple_quote and not inside_block and \
+                   result is not None:
 ##                 print ">> class %r" % result.group(1)
                 if class_name is None:
                     # this is the first class declared in the file: insert the
@@ -199,6 +235,9 @@ class SourceFileContent:
                                      nonce)
                     new_classes_inserted = True
                 class_name = result.group(1)
+                # ALB 2007-08-06: if we're in a subpackage, we should include
+                # the package name in the class name
+                class_name = self.add_package(class_name)
                 self.classes[class_name] = 1 # add the found class to the list
                                              # of classes of this module
                 out_lines.append(line)
@@ -213,6 +252,7 @@ class SourceFileContent:
                     which_class = result.group(2)
                     which_block = result.group(3)
                     if which_class is None: which_class = class_name
+                    else: which_class = self.add_package(which_class)
                     self.spaces[which_class] = spaces
                     inside_block = True
                     if class_name is None:
@@ -226,7 +266,7 @@ class SourceFileContent:
                     result = event_handler.match(line)
                     if not inside_triple_quote and result is not None:
                         which_handler = result.group(1)
-                        which_class = result.group(2)
+                        which_class = self.add_package(result.group(2))
                         self.event_handlers.setdefault(
                             which_class, {})[which_handler] = 1
                     if class_name is not None and self.is_end_of_class(line):
@@ -242,6 +282,7 @@ class SourceFileContent:
             else:
                 # ignore all the lines inside a wxGlade block
                 if block_end.match(line) is not None:
+##                     print 'end block'
                     inside_block = False
         if not new_classes_inserted:
             # if we are here, the previous ``version'' of the file did not
@@ -253,31 +294,54 @@ class SourceFileContent:
         self.content = "".join(out_lines)
 
     def is_import_line(self, line):
+        """True if the line imports wx"""
         if use_new_namespace:
             return line.startswith('import wx')
         else:
             return line.startswith('from wxPython.wx import *')
         
     def is_end_of_class(self, line):
+        """True if the line is the marker for class end"""
         return line.strip().startswith('# end of class ')
+
+    def add_package(self, class_name):
+        if not multiple_files:
+            return class_name
+        name = self.name
+        if out_dir is not None:
+            name = name.replace(out_dir, '')
+        pkg = os.path.dirname(name).replace(os.sep, '.')
+        if pkg.startswith('.'):
+            pkg = pkg[1:]
+        if pkg:
+            return pkg + '.' + class_name
+        else:
+            return class_name
         
 # end of class SourceFileContent
 
-# if not None, it is an instance of SourceFileContent that keeps info about
-# the previous version of the source to generate
 previous_source = None 
+"""\
+If not None, it is an instance of SourceFileContent that keeps info about
+the previous version of the source to generate.
+"""
                   
     
 def tabs(number):
+    """\
+    Returns spaces to indent given number of levels
+    """
     return '    ' * number
 
 
-# if True, overwrite any previous version of the source file instead of
-# updating only the wxGlade blocks
 _overwrite = False
+"""\
+If True, overwrite any previous version of the source file instead of
+updating only the wxGlade blocks.
+"""
 
-# if True, enable gettext support
 _use_gettext = False
+"""If True, enable gettext support"""
 
 
 _quote_str_pattern = re.compile(r'\\[natbv"]?')
@@ -306,17 +370,19 @@ def quote_str(s, translate=True, escape_chars=True):
 def initialize(app_attrs): 
     """\
     Writer initialization function.
-    - app_attrs: dict of attributes of the application. The following two
-                 are always present:
-           path: output path for the generated code (a file if multi_files is
+
+    @param app_attrs: Dict of attributes of the application.
+    
+    The following two are always present:
+      - path: output path for the generated code (a file if multi_files is
                  False, a dir otherwise)
-         option: if True, generate a separate file for each custom class
+      - option: if True, generate a separate file for each custom class
     """
     out_path = app_attrs['path']
     multi_files = app_attrs['option']
 
     global classes, header_lines, multiple_files, previous_source, nonce, \
-           _current_extra_modules, _use_gettext, _overwrite
+           _current_extra_modules, _use_gettext, _overwrite, _current_extra_code
     import time, random
 
     try: _use_gettext = int(app_attrs['use_gettext'])
@@ -356,6 +422,14 @@ def initialize(app_attrs):
                     (common.version, time.asctime(), common.generated_from()),
                     use_new_namespace and 'import wx\n' or
                     'from wxPython.wx import *\n']
+    if not config.preferences.write_timestamp:
+        header_lines[0] = '# generated by wxGlade %s%s\n\n' % \
+                          (common.version, common.generated_from())
+
+
+    # extra lines to generate (see the 'extracode' property of top-level
+    # widgets)
+    _current_extra_code = []
 
     # add coding (PEP 263)
     if _encoding:
@@ -378,7 +452,8 @@ def initialize(app_attrs):
             for line in header_lines:
                 output_file.write(line)
             output_file.write('<%swxGlade extra_modules>\n' % nonce)
-            output_file.write('\n')
+            output_file.write('\n<%swxGlade replace extracode>\n\n' % nonce)
+            #output_file.write('\n')  # PEP 8
     else:
         previous_source = None
         global out_dir
@@ -403,10 +478,17 @@ def finalize():
         tag = '<%swxGlade extra_modules>\n' % nonce
         code = "".join(_current_extra_modules.keys())
         previous_source.content = previous_source.content.replace(tag, code)
+        # extra code (see the 'extracode' property of top-level widgets)
+        tag = '<%swxGlade replace extracode>' % nonce
+        code = "\n".join(['# begin wxGlade: extracode'] + _current_extra_code +
+                         ['# end wxGlade\n'])
+        previous_source.content = previous_source.content.replace(tag, code)
+        
         # now remove all the remaining <123415wxGlade ...> tags from the
         # source: this may happen if we're not generating multiple files,
         # and one of the container class names is changed
-        tags = re.findall('(<%swxGlade replace ([a-zA-Z_]\w*) +\w+>)' % nonce,
+        tags = re.findall('(<%swxGlade replace ([a-zA-Z_]\w*) +[.\w]+>)' % \
+                          nonce,
                           previous_source.content)
         for tag in tags:
             indent = previous_source.spaces.get(tag[1], tabs(2))
@@ -429,6 +511,12 @@ def finalize():
         em = "".join(_current_extra_modules.keys())
         content = output_file.getvalue().replace(
             '<%swxGlade extra_modules>\n' % nonce, em)
+        # extra code (see the 'extracode' property of top-level widgets)
+        tag = '<%swxGlade replace extracode>' % nonce
+        code = "\n".join(['# begin wxGlade: extracode'] + _current_extra_code +
+                         ['# end wxGlade'])
+        content = content.replace(tag, code)
+        
         output_file.close()
         try:
             common.save_file(output_file_name, content, 'codegen')
@@ -463,6 +551,9 @@ def add_object(top_obj, sub_obj):
         klass.init.extend(['\n', '# code for %s (type %s) not generated: '
                            'no suitable writer found' % (sub_obj.name,
                                                          sub_obj.klass),'\n'])
+        common.message('WARNING', 'code for %s (type %s) not generated: '
+                       'no suitable writer found',
+                       sub_obj.name, sub_obj.klass)
     else:
         try:
             init, props, layout = builder.get_code(sub_obj)
@@ -475,6 +566,12 @@ def add_object(top_obj, sub_obj):
                 init.reverse()
                 klass.parents_init.extend(init)
             else: klass.init.extend(init)
+            
+            # Add a dependency of the current object on its parent
+            klass.deps.append((sub_obj, sub_obj.parent))
+            klass.child_order.append(sub_obj)
+            klass.init_lines[sub_obj] = init
+            
             # ---------------------------------------------------------------
             # ALB 2004-12-05
             mycn = getattr(builder, 'cn', cn)
@@ -489,16 +586,47 @@ def add_object(top_obj, sub_obj):
                 for event, handler in sub_obj.properties['events'].iteritems():
                     klass.event_handlers.append((id, mycn(event), handler))
 
+            # try to see if there's some extra code to add to this class
+            if not sub_obj.preview:
+                extra_code = getattr(builder, 'extracode',
+                                     sub_obj.properties.get('extracode', ""))
+                if extra_code:
+                    extra_code = re.sub(r'\\n', '\n', extra_code)
+                    klass.extra_code.append(extra_code)
+                    # if we are not overwriting existing source, warn the user
+                    # about the presence of extra code
+                    if multiple_files:
+                        warn = False
+                    else:
+                        warn = previous_source is not None
+                    if warn:
+                        common.message(
+                            'WARNING',
+                            '%s has extra code, but you are '
+                            'not overwriting existing sources: please check '
+                            'that the resulting code is correct!' % \
+                            sub_obj.name)
+
+
         else: # the object is a sizer
             # ALB 2004-09-17: workaround (hack) for static box sizers...
             if sub_obj.base == 'wxStaticBoxSizer':
-                klass.parents_init.insert(1, init.pop(0))
+                i = init.pop(0)
+                klass.parents_init.insert(1, i)
+                
+                # Add a dependency of the current object on its parent
+                klass.deps.append((sub_obj, sub_obj.parent))
+                klass.child_order.append(sub_obj)
+                klass.init_lines[sub_obj] = [i]
+                
             klass.sizers_init.extend(init)
+            
         klass.props.extend(props)
         klass.layout.extend(layout)
         if multiple_files and \
                (sub_obj.is_toplevel and sub_obj.base != sub_obj.klass):
-            key = 'from %s import %s\n' % (sub_obj.klass, sub_obj.klass)
+            key = 'from %s import %s\n' % (sub_obj.klass,
+                                           without_package(sub_obj.klass))
             klass.dependencies[key] = 1
 ##         for dep in _widget_extra_modules.get(sub_obj.base, []):
         for dep in getattr(obj_builders.get(sub_obj.base),
@@ -531,6 +659,13 @@ def add_sizeritem(toplevel, sizer, obj, option, flag, border):
     klass.layout.append(buffer)
 
 
+def without_package(class_name):
+    """\
+    Removes the package name from the given class name
+    """
+    return class_name.split('.')[-1]
+
+
 def add_class(code_obj):
     """\
     Generates the code for a custom class.
@@ -544,7 +679,7 @@ def add_class(code_obj):
         # let's see if the file to generate exists, and in this case
         # create a SourceFileContent instance
         filename = os.path.join(out_dir,
-                                code_obj.klass.replace('.', '_') + '.py')
+                                code_obj.klass.replace('.', os.sep) + '.py')
         if _overwrite or not os.path.exists(filename): prev_src = None
         else: prev_src = SourceFileContent(filename)
         _current_extra_modules = {}
@@ -566,7 +701,6 @@ def add_class(code_obj):
         # this class wasn't in the previous version of the source (if any)
         is_new = True
         indentation = tabs(2)
-##         mods = _widget_extra_modules.get(code_obj.base)
         mods = getattr(builder, 'extra_modules', [])
         if mods:
             for m in mods: _current_extra_modules[m] = 1
@@ -578,45 +712,107 @@ def add_class(code_obj):
         # if the class body was empty, create an empty ClassLines
         classes[code_obj.klass] = ClassLines()
 
-##     # first thing to do, call the property writer: we do this now because it
-##     # can have side effects that modify the ClassLines instance (this is used
-##     # in the toplevel menubar)
-##     props_builder = obj_properties.get(code_obj.base)
-##     write_body = len(classes[code_obj.klass].props)
-##     if props_builder:
-##         obj_p = obj_properties[code_obj.base](code_obj)
-##         if not write_body: write_body = len(obj_p)
-##     else: obj_p = []
+    # try to see if there's some extra code to add to this class
+    if not code_obj.preview:
+        extra_code = getattr(builder, 'extracode',
+                             code_obj.properties.get('extracode', ""))
+        if extra_code:
+            extra_code = re.sub(r'\\n', '\n', extra_code)
+            classes[code_obj.klass].extra_code.append(extra_code)
+            if not is_new:
+                common.message('WARNING', '%s has extra code, but you are '
+                               'not overwriting existing sources: please check '
+                               'that the resulting code is correct!' % \
+                               code_obj.name)
 
+        # Don't add extra_code to _current_extra_code here, that is handled
+        # later.  Otherwise we'll emit duplicate extra code for frames.
+
+    # ALB 2007-08-31 custom base classes support
+    custom_base = getattr(code_obj, 'custom_base',
+                          code_obj.properties.get('custom_base', None))
+    if code_obj.preview or (custom_base and not custom_base.strip()):
+        custom_base = None
+        
     if is_new:
         base = mycn(code_obj.base)
+        if custom_base is not None:
+            base = ", ".join([b.strip() for b in custom_base.split(',')])
         if code_obj.preview and code_obj.klass == base:
             import random
             klass = code_obj.klass + ('_%d' % random.randrange(10**8, 10**9))
         else:
             klass = code_obj.klass
-        write('class %s(%s):\n' % (klass, base))
+        write('\nclass %s(%s):\n' % (without_package(klass), base))
         write(tabs(1) + 'def __init__(self, *args, **kwds):\n')
+    elif custom_base is not None:
+        # custom base classes set, but "overwrite existing sources" not
+        # set. Issue a warning about this
+        common.message('WARNING', '%s has custom base classes, but you are '
+                       'not overwriting existing sources: please check that '
+                       'the resulting code is correct!' % code_obj.name)
+                       
     # __init__ begin tag
-    write(indentation + '# begin wxGlade: %s.__init__\n' % code_obj.klass)
+    write(indentation + '# begin wxGlade: %s.__init__\n' % \
+          without_package(code_obj.klass))
     prop = code_obj.properties
     style = prop.get("style", None)
     if style: write(indentation + 'kwds["style"] = %s\n' % mycn_f(style))
     # __init__
-    write(indentation + '%s.__init__(self, *args, **kwds)\n' % \
-          mycn(code_obj.base))
+    if custom_base is not None:
+        bases = [b.strip() for b in custom_base.split(',')]
+        for i, b in enumerate(bases):
+            if not i:
+                write(indentation + '%s.__init__(self, *args, **kwds)\n' % b)
+            else:
+                write(indentation + '%s.__init__(self)\n' % b)
+    else:
+        write(indentation + '%s.__init__(self, *args, **kwds)\n' % \
+              mycn(code_obj.base))
     tab = indentation 
     init_lines = classes[code_obj.klass].init
     # --- patch 2002-08-26 ---------------------------------------------------
     parents_init = classes[code_obj.klass].parents_init
-    parents_init.reverse()
-    for l in parents_init: write(tab+l)
+
+
+    # classes[code_obj.klass].deps now contains a mapping of child to parent 
+    # For all children we processed...
+    object_order = []
+    for obj in classes[code_obj.klass].child_order:
+        # Don't add it again if already present
+        if obj in object_order:
+            continue
+        
+        object_order.append(obj)
+        
+        # Insert parent and ancestor objects before the current object
+        current_object = obj
+        for child, parent in classes[code_obj.klass].deps[:]:
+            if child is current_object:
+                if parent not in object_order:
+                    idx = object_order.index(current_object)
+                    object_order.insert(idx, parent)
+                current_object = parent
+                
+                # We processed the dependency: remove it
+                classes[code_obj.klass].deps.remove((child, parent))
+    
+    #for l in parents_init: write(tab+l)
+    
+    # Write out the initialisation in the order we just generated 
+    for obj in object_order:
+        if obj in classes[code_obj.klass].init_lines:
+            for l in classes[code_obj.klass].init_lines[obj]:
+                write(tab + l)
+
+    
     # ------------------------------------------------------------------------
-    for l in init_lines: write(tab + l)
+    #for l in init_lines: write(tab + l)
 
     # now check if there are extra lines to add to the init method
     if hasattr(builder, 'get_init_code'):
-        for l in builder.get_init_code(code_obj): write(tab + l)
+        for l in builder.get_init_code(code_obj): 
+            write(tab + l)
     
     write('\n' + tab + 'self.__set_properties()\n')
     write(tab + 'self.__do_layout()\n')
@@ -650,20 +846,15 @@ def add_class(code_obj):
                                              '__init__')
         if prev_src.content.find(tag) < 0:
             # no __init__ tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade __init__ block not found," \
-                  " __init__ code NOT generated"
+            common.message("WARNING",
+                           "wxGlade __init__ block not found for %s," \
+                           " __init__ code NOT generated" % code_obj.name)
         else:
             prev_src.content = prev_src.content.replace(tag, "".join(buffer))
         buffer = []
         write = buffer.append
 
     # __set_properties
-##     props_builder = obj_properties.get(code_obj.base)
-##     write_body = len(classes[code_obj.klass].props)
-##     if props_builder:
-##         obj_p = obj_properties[code_obj.base](code_obj)
-##         if not write_body: write_body = len(obj_p)
-##     else: obj_p = []
     obj_p = getattr(builder, 'get_properties_code',
                     generate_common_properties)(code_obj)
     obj_p.extend(classes[code_obj.klass].props)
@@ -671,7 +862,8 @@ def add_class(code_obj):
 
     if is_new: write('\n%sdef __set_properties(self):\n' % tabs(1))
     # begin tag
-    write(tab + '# begin wxGlade: %s.__set_properties\n' % code_obj.klass)
+    write(tab + '# begin wxGlade: %s.__set_properties\n' % \
+          without_package(code_obj.klass))
     if not write_body: write(tab + 'pass\n')
     else:
         for l in obj_p: write(tab + l)
@@ -684,8 +876,10 @@ def add_class(code_obj):
                                              '__set_properties')
         if prev_src.content.find(tag) < 0:
             # no __set_properties tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade __set_properties block " \
-                  "not found, __set_properties code NOT generated"
+            common.message("WARNING", "wxGlade __set_properties block " \
+                           "not found for %s, "
+                           "__set_properties code NOT generated" %
+                           code_obj.name)
         else:
             prev_src.content = prev_src.content.replace(tag, "".join(buffer))
         buffer = []
@@ -703,7 +897,8 @@ def add_class(code_obj):
         extra_layout_lines = []
     
     # begin tag
-    write(tab + '# begin wxGlade: %s.__do_layout\n' % code_obj.klass)
+    write(tab + '# begin wxGlade: %s.__do_layout\n' % \
+          without_package(code_obj.klass))
     if layout_lines or sizers_init_lines or extra_layout_lines:
         sizers_init_lines.reverse()
         for l in sizers_init_lines: write(tab + l)
@@ -720,8 +915,9 @@ def add_class(code_obj):
                                              '__do_layout')
         if prev_src.content.find(tag) < 0:
             # no __do_layout tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade __do_layout block " \
-                  "not found, __do_layout code NOT generated"
+            common.message("WARNING", "wxGlade __do_layout block " \
+                           "not found for %s, __do_layout code NOT generated" %
+                           code_obj.name)
         else:
             prev_src.content = prev_src.content.replace(tag, "".join(buffer))
 
@@ -731,9 +927,9 @@ def add_class(code_obj):
         buf = []
         for name, event, handler in event_handlers:
             if handler not in already_there:
-                buf.append(tabs(1) + 'def %s(self, event): '
+                buf.append(tabs(1) + 'def %s(self, event):  '
                            '# wxGlade: %s.<event_handler>\n'
-                           % (handler, code_obj.klass))
+                           % (handler, without_package(code_obj.klass)))
                 buf.append(
                     tab + 'print "Event handler `%s\' not implemented"\n' %
                     handler)
@@ -742,8 +938,9 @@ def add_class(code_obj):
         tag = '<%swxGlade event_handlers %s>' % (nonce, code_obj.klass)
         if prev_src.content.find(tag) < 0:
             # no event_handlers tag found, issue a warning and do nothing
-            print >> sys.stderr, "WARNING: wxGlade event_handlers block " \
-                  "not found, event_handlers code NOT generated"
+            common.message("WARNING", "wxGlade event_handlers block " \
+                           "not found for %s,"
+                           " event_handlers code NOT generated" % code_obj.name)
         else:
             prev_src.content = prev_src.content.replace(tag, "".join(buf))
         del buf
@@ -751,9 +948,9 @@ def add_class(code_obj):
         already_there = {}
         for name, event, handler in event_handlers:
             if handler not in already_there:
-                write('\n' + tabs(1) + 'def %s(self, event): '
+                write('\n' + tabs(1) + 'def %s(self, event):  '
                       '# wxGlade: %s.<event_handler>\n'
-                      % (handler, code_obj.klass))
+                      % (handler, without_package(code_obj.klass)))
                 write(tab + 'print "Event handler `%s\' not implemented!"\n' %
                       handler)
                 write(tab + 'event.Skip()\n')
@@ -762,7 +959,7 @@ def add_class(code_obj):
     # the code has been generated
     classes[code_obj.klass].done = True
 
-    write('\n# end of class %s\n\n\n' % code_obj.klass)
+    write('\n# end of class %s\n' % without_package(code_obj.klass))
 
     if not multiple_files and prev_src is not None:
         # if this is a new class, add its code to the new_classes list of the
@@ -786,7 +983,24 @@ def add_class(code_obj):
                    ['# end wxGlade\n']
             tag = '<%swxGlade replace dependencies>' % nonce
             prev_src.content = prev_src.content.replace(tag, "".join(deps))
+
+            # insert the extra code of this class
+            extra_code = "".join(classes[code_obj.klass].extra_code[::-1])
+            # if there's extra code but we are not overwriting existing
+            # sources, warn the user
+            if extra_code:
+                common.message('WARNING', '%s (or one of its chilren) has '
+                               'extra code classes, but you are '
+                               'not overwriting existing sources: please check '
+                               'that the resulting code is correct!' % \
+                               code_obj.name)
             
+            extra_code = '# begin wxGlade: extracode\n%s\n# end wxGlade\n' % \
+                         extra_code
+##                          "".join(classes[code_obj.klass].extra_code[::-1])
+            tag = '<%swxGlade replace extracode>' % nonce
+            prev_src.content = prev_src.content.replace(tag, extra_code)
+
             try:
                 # store the new file contents to disk
                 common.save_file(filename, prev_src.content, 'codegen')
@@ -796,7 +1010,8 @@ def add_class(code_obj):
             return
 
         # create the new source file
-        filename = os.path.join(out_dir, code_obj.klass + '.py')
+        filename = os.path.join(out_dir,
+                                code_obj.klass.replace('.', os.sep) + '.py')
         out = cStringIO.StringIO()
         write = out.write
         # write the common lines
@@ -808,6 +1023,13 @@ def add_class(code_obj):
             write(module)
         write('# end wxGlade\n')
         write('\n')
+
+        # insert the extra code of this class
+        extra_code = "".join(classes[code_obj.klass].extra_code[::-1])
+        extra_code = '# begin wxGlade: extracode\n%s\n# end wxGlade\n' % \
+                     extra_code
+        write(extra_code)
+        write('\n')
         
         # write the class body
         for line in buffer: write(line)
@@ -821,6 +1043,8 @@ def add_class(code_obj):
         # write the class body onto the single source file
         for dep in classes[code_obj.klass].dependencies:
             _current_extra_modules[dep] = 1
+        if classes[code_obj.klass].extra_code:
+            _current_extra_code.extend(classes[code_obj.klass].extra_code[::-1])
         write = output_file.write
         for line in buffer: write(line)
         
@@ -1022,6 +1246,9 @@ def generate_code_tooltip(obj):
 
 
 def generate_code_disabled(obj):
+    """\
+    returns the code fragment that disables the given object.
+    """
     self = _get_code_name(obj)
     try: disabled = int(obj.properties['disabled'])
     except: disabled = False
@@ -1030,6 +1257,9 @@ def generate_code_disabled(obj):
 
 
 def generate_code_focused(obj):
+    """\
+    returns the code fragment that get the focus to the given object.
+    """
     self = _get_code_name(obj)
     try: focused = int(obj.properties['focused'])
     except: focused = False
@@ -1038,12 +1268,27 @@ def generate_code_focused(obj):
 
 
 def generate_code_hidden(obj):
+    """\
+    returns the code fragment that hides the given object.
+    """
     self = _get_code_name(obj)
     try: hidden = int(obj.properties['hidden'])
     except: hidden = False
     if hidden:
         return self + '.Hide()\n'
-    
+
+
+def generate_code_extraproperties(obj):
+    """\
+    returns the code fragment that set extra properties for the given object.
+    """
+    self = _get_code_name(obj)
+    prop = obj.properties['extraproperties']
+    ret = []
+    for name in sorted(prop):
+        ret.append(self + '.Set%s(%s)\n' % (name, prop[name]))
+    return ret
+
 
 def generate_common_properties(widget):
     """\
@@ -1063,6 +1308,9 @@ def generate_common_properties(widget):
     if prop.get('disabled'): out.append(generate_code_disabled(widget))
     if prop.get('focused'): out.append(generate_code_focused(widget))
     if prop.get('hidden'): out.append(generate_code_hidden(widget))
+    # ALB 2007-09-01 extra properties
+    if prop.get('extraproperties') and not widget.preview:
+        out.extend(generate_code_extraproperties(widget))
     return out
 
 
@@ -1143,24 +1391,66 @@ class EventsPropertyHandler(object):
             self.curr_handler.append(data)
 
 # end of class EventsPropertyHandler
+
+
+class ExtraPropertiesPropertyHandler(object):
+    def __init__(self):
+        self.props = {}
+        self.prop_name = None
+        self.curr_prop = []
+        
+    def start_elem(self, name, attrs):
+        if name == 'property':
+            name = attrs['name']
+            if name and name[0].islower():
+                name = name[0].upper() + name[1:]
+            self.prop_name = name
+
+    def end_elem(self, name, code_obj):
+        if name == 'property':
+            if self.prop_name and self.curr_prop:
+                self.props[self.prop_name] = ''.join(self.curr_prop)
+            self.prop_name = None
+            self.curr_prop = []
+        elif name == 'extraproperties':
+            code_obj.properties['extraproperties'] = self.props
+            return True # to remove this handler
+
+    def char_data(self, data):
+        data = data.strip()
+        if data:
+            self.curr_prop.append(data)
+
+# end of class ExtraPropertiesPropertyHandler
     
 
 # dictionary whose items are custom handlers for widget properties
 _global_property_writers = { 'font': FontPropertyHandler,
-                             'events': EventsPropertyHandler, }
+                             'events': EventsPropertyHandler,
+                             'extraproperties': ExtraPropertiesPropertyHandler,
+                             }
 
-# dictionary of dictionaries of property handlers specific for a widget
-# the keys are the class names of the widgets
-# Ex: _property_writers['wxRadioBox'] = {'choices', choices_handler}
 _property_writers = {}
+"""\
+Dictionary of dictionaries of property handlers specific for a widget
+the keys are the class names of the widgets
 
+Example: _property_writers['wxRadioBox'] = {'choices', choices_handler}
+"""
 
-# map of widget class names to a list of extra modules needed for the
-# widget. Example: 'wxGrid': 'from wxPython.grid import *\n'
 _widget_extra_modules = {}
+"""\
+Map of widget class names to a list of extra modules needed for the
+widget.
+
+Example: 'wxGrid': 'from wxPython.grid import *\n'
+"""
+
+_current_extra_code = []
+"""Set of lines for extra code to add to the current file"""
 
-# set of lines of extra modules to add to the current file
 _current_extra_modules = {} 
+"""Set of lines of extra modules to add to the current file"""
 
 def get_property_handler(property_name, widget_name):
     try: cls = _property_writers[widget_name][property_name]
@@ -1186,8 +1476,10 @@ class WidgetHandler:
     Interface the various code generators for the widgets must implement
     """
     
-    """list of modules to import (eg. ['from wxPython.grid import *\n'])"""
     import_modules = []
+    """\
+    List of modules to import (eg. ['from wxPython.grid import *\n'])
+    """
 
     def get_code(self, obj):
         """\
