@@ -1,7 +1,7 @@
 # edit_sizers.py: hierarchy of Sizers supported by wxGlade
-# $Id: edit_sizers.py,v 1.80 2007/08/07 12:21:55 agriggio Exp $
 # 
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
+#
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
@@ -21,8 +21,7 @@ class SizerSlot:
         self.menu = None
 
     def create_widget(self):
-        if misc.check_wx_version(2, 5, 2): style = wx.FULL_REPAINT_ON_RESIZE
-        else: style = 0
+        style = wx.FULL_REPAINT_ON_RESIZE
         self.widget = wx.Window(self.parent.widget, -1, size=(20, 20),
                                style=style)
         self.widget.SetBackgroundColour(wx.LIGHT_GREY)
@@ -40,7 +39,7 @@ class SizerSlot:
             evt_key = event.GetKeyCode()
             for flags, key, function in misc.accel_table:
                 if evt_flags == flags and evt_key == key:
-                    misc.wxCallAfter(function)
+                    wx.CallAfter(function)
                     break
             #event.Skip()
         wx.EVT_KEY_DOWN(self.widget, on_key_down)
@@ -89,7 +88,7 @@ class SizerSlot:
             misc.append_item(self.menu, PASTE_ID, _('Paste\tCtrl+V'),
                              wx.ART_PASTE)
             def bind(method):
-                return lambda e: misc.wxCallAfter(method)
+                return lambda e: wx.CallAfter(method)
             wx.EVT_MENU(self.widget, REMOVE_ID, bind(self.remove))
             wx.EVT_MENU(self.widget, PASTE_ID, bind(self.clipboard_paste))
 
@@ -188,8 +187,10 @@ class SizerHandleButton(Button):
         self.sizer = sizer
         self.menu = menu
         self._rmenu = None
-        try: self.SetUseFocusIndicator(False)
-        except AttributeError: pass
+        try:
+            self.SetUseFocusIndicator(False)
+        except AttributeError:
+            pass
 ##         # provide popup menu for removal
 ##         REMOVE_ID = wxNewId() 
 ##         self._rmenu = misc.wxGladePopupMenu(sizer.name)
@@ -216,7 +217,7 @@ class SizerHandleButton(Button):
             evt_key = event.GetKeyCode()
             for flags, key, function in misc.accel_table:
                 if evt_flags == flags and evt_key == key:
-                    misc.wxCallAfter(function)
+                    wx.CallAfter(function)
                     break
             #event.Skip()
         wx.EVT_KEY_DOWN(self, on_key_down)
@@ -235,7 +236,7 @@ class SizerHandleButton(Button):
             REMOVE_ID = wx.NewId() 
             self._rmenu = misc.wxGladePopupMenu(self.sizer.name)
             def bind(method):
-                return lambda e: misc.wxCallAfter(method)
+                return lambda e: wx.CallAfter(method)
             #self._rmenu.Append(REMOVE_ID, 'Remove\tDel')
             misc.append_item(self._rmenu, REMOVE_ID, _('Remove\tDel'),
                              wx.ART_DELETE)
@@ -638,7 +639,7 @@ class SizerBase(Sizer):
         return self.pos - 1
     
     def set_pos(self, value):
-        misc.wxCallAfter(self.sizer.change_item_pos, 
+        wx.CallAfter(self.sizer.change_item_pos, 
                          self, min(value + 1, len(self.sizer.children) - 1))
 
     def update_pos(self, value):
@@ -647,19 +648,14 @@ class SizerBase(Sizer):
         self.pos = value
 
     def change(self, *args):
-        # if wxPython < 2.3.3, wxCallAfter is defined in misc.py
-        misc.wxCallAfter(change_sizer, self, self.klass_prop.get_value())
+        wx.CallAfter(change_sizer, self, self.klass_prop.get_value())
 
     def create_properties(self):
         """\
         Displays the Properties of self
         """
         self.notebook = wx.Notebook(common.property_panel, -1)
-        if not misc.check_wx_version(2, 5, 2):
-            nb_sizer = wx.NotebookSizer(self.notebook)
-            self.notebook.sizer = nb_sizer
-        else:
-            self.notebook.sizer = None
+        self.notebook.sizer = None
         self.notebook.SetAutoLayout(True)
         panel = wx.ScrolledWindow(self.notebook, -1, style=wx.TAB_TRAVERSAL)
         sizer_tmp = wx.BoxSizer(wx.VERTICAL)
@@ -704,7 +700,7 @@ class SizerBase(Sizer):
         value = "%s" % value
         if not config.preferences.allow_duplicate_names and \
                (self.widget and common.app_tree.has_name(value, self.node)):
-            misc.wxCallAfter(
+            wx.CallAfter(
                 wx.MessageBox, _('Name "%s" is already in use.\n'
                 'Please enter a different one.') % value, _("Error"),
                 wx.OK|wx.ICON_ERROR)
@@ -716,9 +712,11 @@ class SizerBase(Sizer):
             oldname = self.name
             self.name = value
             self._btn.set_menu_title(value)
-            try: common.app_tree.refresh_name(self.node, oldname) #, self.name)
+            try:
+                common.app_tree.refresh_name(self.node, oldname)
             except AttributeError:
-                import traceback; traceback.print_exc()
+                import traceback
+                traceback.print_exc()
             self.property_window.SetTitle(_('Properties - <%s>') % self.name)
     set_name_pattern = re.compile('^[a-zA-Z_]+[\w0-9]*$')
             
@@ -764,8 +762,10 @@ class SizerBase(Sizer):
         self.property_window.Layout()
         self.property_window.SetTitle(_('Properties - <%s>') % self.name)
         if hasattr(self, 'node'): common.app_tree.select_item(self.node)
-        try: self._btn.SetFocus()
-        except AttributeError: pass
+        try:
+            self._btn.SetFocus()
+        except AttributeError:
+            pass
         
     def fit_parent(self, *args):
         """\
@@ -794,7 +794,8 @@ class SizerBase(Sizer):
             self.children[pos] = SizerItem(item, pos, option, flag, border,
                                            size)
         except IndexError: # this shouldn't happen!
-            import traceback; traceback.print_exc()
+            import traceback
+            traceback.print_exc()
             print self.children, pos
             raise SystemExit
 
@@ -828,25 +829,12 @@ class SizerBase(Sizer):
             self.widget.SetItemMinSize(item.widget, w, h)
             return
 
-        if not misc.check_wx_version(2, 5):
-            if elem.IsWindow(): # remove the previous item at pos
-                w = elem.GetWindow()
-                elem.SetWindow(None)
-                w.Destroy()
-            try: # let's see if the item to add is a window
-                elem.SetWindow(item.widget)
-            except TypeError: # suppose the item to add is a sizer
-                elem.SetSizer(item.widget)
-            elem.SetOption(option)
-            elem.SetFlag(flag)
-            elem.SetBorder(border)
-        else:
-            self.widget.Insert(pos, item.widget, option, flag, border)
-            self.widget.Remove(pos+1)
-            if elem.IsWindow():
-                w = elem.GetWindow()
-                w.SetContainingSizer(None)
-                w.Destroy()
+        self.widget.Insert(pos, item.widget, option, flag, border)
+        self.widget.Remove(pos+1)
+        if elem.IsWindow():
+            w = elem.GetWindow()
+            w.SetContainingSizer(None)
+            w.Destroy()
                 
         try: # if the item was a window, set its size to a reasonable value
 ##         if elem.IsWindow():
@@ -856,8 +844,7 @@ class SizerBase(Sizer):
             if h == -1: h = item.widget.GetBestSize()[1]
 
             option = 0
-            if misc.check_wx_version(2, 5): option = elem.GetProportion()
-            else: option = elem.GetOption()
+            option = elem.GetProportion()
             flag = elem.GetFlag()
             if not size or (option == 0 or not (flag & wx.EXPAND)):
                 self.widget.SetItemMinSize(item.widget, w, h)
@@ -866,7 +853,7 @@ class SizerBase(Sizer):
                 self.widget.SetItemMinSize(item.widget, w, h)
                 #*item.widget.GetBestSize())
             #self.widget.SetItemMinSize(item.widget, w, h)
-        except Exception, e:
+        except Exception:
             #import traceback; traceback.print_exc()
             pass
         if force_layout: self.layout() # update the layout of self
@@ -882,12 +869,9 @@ class SizerBase(Sizer):
         # no error checking at all, this is a "protected" method, so it should
         # be safe to assume the caller knows how to use it
         item = self.widget.GetChildren()[pos]
-        if not misc.check_wx_version(2, 5):
-            item.SetWindow(None)
-        else:
-            if item.IsWindow():
-                w = item.GetWindow()
-                w.SetContainingSizer(None)
+        if item.IsWindow():
+            w = item.GetWindow()
+            w.SetContainingSizer(None)
         item.SetSizer(notebook_sizer)
         if force_layout:
             self.layout()
@@ -897,7 +881,8 @@ class SizerBase(Sizer):
         """\
         Updates the layout of the item at the given pos.
         """
-        try: item = self.children[pos]
+        try:
+            item = self.children[pos]
         except IndexError: # this shouldn't happen
             import traceback; traceback.print_exc()
             raise SystemExit
@@ -917,14 +902,13 @@ class SizerBase(Sizer):
     def _set_item_widget(self, pos, option, flag, border, size, force_layout):
         if not self.widget: return
         
-        try: elem = self.widget.GetChildren()[pos]
-        except IndexError: return # this may happen during xml loading
+        try:
+            elem = self.widget.GetChildren()[pos]
+        except IndexError:
+            return # this may happen during xml loading
 
         if option is not None:
-            if not misc.check_wx_version(2, 5):
-                elem.SetOption(option)
-            else:
-                elem.SetProportion(option)
+            elem.SetProportion(option)
         if flag is not None:
             elem.SetFlag(flag)
         if border is not None:
@@ -939,10 +923,7 @@ class SizerBase(Sizer):
             newelem.SetWindow(item)
             newelem.SetFlag(elem.GetFlag())
             newelem.SetBorder(elem.GetBorder())
-            if misc.check_wx_version(2, 5):
-                newelem.SetProportion(elem.GetProportion())
-            else:
-                newelem.SetOption(elem.GetOption())
+            newelem.SetProportion(elem.GetProportion())
             newelem.SetInitSize(w, h)
             self.widget.InsertItem(pos, newelem)
             #self.children[pos] = newelem
@@ -990,8 +971,7 @@ class SizerBase(Sizer):
                               self.window.widget.GetId())
             wx.PostEvent(self.window.widget, evt)
             # don't change the size of the window
-            if misc.check_wx_version(2, 4, 1) and \
-                   not misc.check_wx_version(2, 6, 0):
+            if not misc.check_wx_version(2, 6, 0):
                 # this seems to work bad for 2.4.0 (and 2.6 too... 2005-05-01)
                 self.widget.FitInside(self.window.widget)
             return
@@ -1000,7 +980,8 @@ class SizerBase(Sizer):
         for c in self.children:
             try:
                 c.item.widget.Refresh()
-            except Exception, e: pass
+            except Exception:
+                pass
         if recursive:
             if getattr(self, 'sizer', None) is not None:
                 self.sizer.layout(recursive)
@@ -1061,7 +1042,8 @@ class SizerBase(Sizer):
         try:
             self.sizer.set_item(self.pos, option=self.option)
             #print self.name, 'set_option', self.option
-        except AttributeError, e: pass
+        except AttributeError:
+            pass
         self.finish_set()
 
     def set_flag(self, value):
@@ -1075,8 +1057,10 @@ class SizerBase(Sizer):
             if value[v]:
                 flags |= self.flags_pos[v]
         self.flag = flags
-        try: self.sizer.set_item(self.pos, flag=flags)
-        except AttributeError, e: pass
+        try:
+            self.sizer.set_item(self.pos, flag=flags)
+        except AttributeError:
+            pass
         self.finish_set()
 
     def set_border(self, value):
@@ -1085,8 +1069,10 @@ class SizerBase(Sizer):
         value of the border property
         """
         self.border = int(value)
-        try: self.sizer.set_item(self.pos, border=self.border)
-        except AttributeError, e: print e
+        try:
+            self.sizer.set_item(self.pos, border=self.border)
+        except AttributeError, e:
+            print e
 
     def get_option(self):
         if not hasattr(self, 'sizer'): return '1'
@@ -1104,15 +1090,19 @@ class SizerBase(Sizer):
                 retval[0] = 1; retval[1:5] = [0, 0, 0, 0]
             else:
                 retval[0] = 0
-        except AttributeError: pass
+        except AttributeError:
+            pass
         return retval
 
     def get_int_flag(self):
-        try: return self.flag
-        except AttributeError: return wx.EXPAND
+        try:
+            return self.flag
+        except AttributeError:
+            return wx.EXPAND
 
     def get_border(self):
-        if not hasattr(self, 'sizer'): return '0'
+        if not hasattr(self, 'sizer'):
+            return '0'
         return str(self.border)
 
     def remove(self):
@@ -1147,8 +1137,10 @@ class SizerBase(Sizer):
         def finish_set(self):
             for c in self.children:
                 if c.item.widget:
-                    try: c.item.widget.Refresh()
-                    except AttributeError: pass # sizers have no Refresh
+                    try:
+                        c.item.widget.Refresh()
+                    except AttributeError:
+                        pass # sizers have no Refresh
     else:
         def finish_set(self): pass
 
@@ -1156,8 +1148,10 @@ class SizerBase(Sizer):
         # this will be self.widget.Refresh
         for c in self.children:
             if c.item.widget: 
-                try: c.item.widget.Refresh()
-                except AttributeError: pass
+                try:
+                    c.item.widget.Refresh()
+                except AttributeError:
+                    pass
 
     def update_view(self, selected):
         if self._btn is not None:
@@ -1215,19 +1209,8 @@ class SizerBase(Sizer):
         self.children[pos] = item
         if self.widget:
             tmp.show_widget(True) # create the actual SizerSlot
-            if misc.check_wx_version(2, 6):
-                self.widget.Insert(pos+1, tmp.widget, 1, wx.EXPAND)
-                self.widget.Detach(pos)
-            else:
-                elem = self.widget.GetChildren()[pos]
-                elem.SetWindow(tmp.widget)
-                elem.SetSizer(None)
-                if not misc.check_wx_version(2, 5):
-                    elem.SetOption(1)
-                else:
-                    elem.SetProportion(1)
-                elem.SetBorder(0)
-                elem.SetFlag(wx.EXPAND)
+            self.widget.Insert(pos+1, tmp.widget, 1, wx.EXPAND)
+            self.widget.Detach(pos)
             if force_layout: self.layout()
 
     def is_visible(self):
@@ -1316,17 +1299,12 @@ class EditBoxSizer(SizerBase):
                 sp = c.item.properties.get('size')
                 if sp and sp.is_active():
                     if (c.option != 0 or (c.flag & wx.EXPAND)) and \
-                           (misc.check_wx_version(2, 4) or \
-                            not (c.flag & wx.FIXED_MINSIZE)):
+                           not (c.flag & wx.FIXED_MINSIZE):
                         c.item.widget.Layout()
                         w, h = c.item.widget.GetBestSize()
-                        if misc.check_wx_version(2, 5):
-                            #print "HERE:", w, h
-                            c.item.widget.SetMinSize((w, h))
+                        #print "HERE:", w, h
+                        c.item.widget.SetMinSize((w, h))
                     else:
-##                     if misc.check_wx_version(2, 4) or \
-##                            ((c.flag & wxFIXED_MINSIZE) and \
-##                             (c.option == 0 or not (c.flag & wxEXPAND))):
                         size = sp.get_value().strip()
                         if size[-1] == 'd':
                             size = size[:-1]
@@ -1335,13 +1313,8 @@ class EditBoxSizer(SizerBase):
                         w, h = [ int(v) for v in size.split(',') ]
                         if use_dialog_units:
                             w, h = wx.DLG_SZE(c.item.widget, (w, h))
-##                     else:
-##                         w, h = c.item.widget.GetBestSize()
                         # now re-set the item to update the size correctly...
                         to_lay_out.append((c.item.pos, (w, h)))
-##                 else:
-##                     w, h = c.item.widget.GetBestSize()
-##                 self.widget.SetItemMinSize(c.item.widget, w, h)
         for pos, size in to_lay_out:
             self.set_item(pos, size=size, force_layout=False)
         self.layout(True)
@@ -1581,12 +1554,10 @@ class GridSizerBase(SizerBase):
                 sp = c.item.properties.get('size')
                 if sp and sp.is_active():
                     if (c.option != 0 or (c.flag & wx.EXPAND)) and \
-                           (misc.check_wx_version(2, 4) or \
-                            not (c.flag & wx.FIXED_MINSIZE)):
+                           not (c.flag & wx.FIXED_MINSIZE):
                         c.item.widget.Layout()
                         w, h = c.item.widget.GetBestSize()
-                        if misc.check_wx_version(2, 5):
-                            c.item.widget.SetMinSize((w, h))
+                        c.item.widget.SetMinSize((w, h))
                     else:
                         size = sp.get_value().strip()
                         if size[-1] == 'd':
@@ -1794,14 +1765,13 @@ class GridSizerBase(SizerBase):
     def _set_item_widget(self, pos, option, flag, border, size, force_layout):
         if not self.widget: return
         
-        try: elem = self.widget.GetChildren()[pos]
-        except IndexError: return # this may happen during xml loading
+        try:
+            elem = self.widget.GetChildren()[pos]
+        except IndexError:
+            return # this may happen during xml loading
 
         if option is not None:
-            if not misc.check_wx_version(2, 5):
-                elem.SetOption(option)
-            else:
-                elem.SetProportion(option)
+            elem.SetProportion(option)
         if flag is not None:
             elem.SetFlag(flag)
         if border is not None:
@@ -1965,7 +1935,8 @@ class EditFlexGridSizer(GridSizerBase):
         sizer.Fit(page)
         
     def set_growable_rows(self, value):
-        try: self.grow_rows = [int(i) for i in value.split(',')]
+        try:
+            self.grow_rows = [int(i) for i in value.split(',')]
         except:
             if not value.strip(): self.grow_rows = []
             else:
@@ -1975,11 +1946,12 @@ class EditFlexGridSizer(GridSizerBase):
         if self.widget:
             if self.notebook: page = self.notebook.GetSelection()
             else: page = 0
-            misc.wxCallAfter(change_sizer, self, self.klass_prop.get_value(),
+            wx.CallAfter(change_sizer, self, self.klass_prop.get_value(),
                              page)
 
     def set_growable_cols(self, value):
-        try: self.grow_cols = [int(i) for i in value.split(',')]
+        try:
+            self.grow_cols = [int(i) for i in value.split(',')]
         except:
             if not value.strip(): self.grow_cols = []
             else:
@@ -1989,7 +1961,7 @@ class EditFlexGridSizer(GridSizerBase):
         if self.widget:
             if self.notebook: page = self.notebook.GetSelection()
             else: page = 0
-            misc.wxCallAfter(change_sizer, self, self.klass_prop.get_value(),
+            wx.CallAfter(change_sizer, self, self.klass_prop.get_value(),
                              page)
 
     def get_growable_rows(self):
@@ -2122,8 +2094,10 @@ def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
     factory function to build EditBoxSizer objects from an xml file
     """
     from xml_parse import XmlParsingError
-    try: name = attrs['name']
-    except KeyError: raise XmlParsingError, _("'name' attribute missing")
+    try:
+        name = attrs['name']
+    except KeyError:
+        raise XmlParsingError, _("'name' attribute missing")
     orientation = wx.VERTICAL # default value
     if sizer is not None: topl = False
     else: topl = True
@@ -2238,8 +2212,10 @@ def grid_xml_builder(attrs, parent, sizer, sizeritem, pos=None):
     factory function to build EditGridSizer objects from an xml file
     """
     from xml_parse import XmlParsingError
-    try: name = attrs['name']
-    except KeyError: raise XmlParsingError, _("'name' attribute missing")
+    try:
+        name = attrs['name']
+    except KeyError:
+        raise XmlParsingError, _("'name' attribute missing")
     if attrs['base'] == 'EditGridSizer': constructor = EditGridSizer
     else: constructor = EditFlexGridSizer
     if sizer is not None: 
@@ -2277,10 +2253,14 @@ def init_all():
     cwx['EditFlexGridSizer'] = grid_xml_builder
 
     import os.path
-    WidgetTree.images['EditStaticBoxSizer'] = os.path.join(common.wxglade_path,
-                                                           'icons/sizer.xpm')
+    WidgetTree.images['EditStaticBoxSizer'] = os.path.join(
+        common.icons_path,
+        'sizer.xpm'
+        )
     WidgetTree.images['EditFlexGridSizer'] = os.path.join(
-        common.wxglade_path, 'icons/grid_sizer.xpm')
+        common.icons_path,
+        'grid_sizer.xpm'
+        )
 
     return [common.make_object_button('EditBoxSizer', 'icons/sizer.xpm'),
             common.make_object_button('EditGridSizer', 'icons/grid_sizer.xpm')]
