@@ -1,13 +1,16 @@
 # tree.py: classes to handle and display the structure of a wxGlade app
-# $Id: tree.py,v 1.56 2007/08/07 12:13:44 agriggio Exp $
 # 
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
+#
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
+import os.path
 import wx
+
 from xml.sax.saxutils import quoteattr
-import misc, common, os.path
+import misc
+import common
 
 try: set
 except NameError: from sets import Set as set
@@ -206,7 +209,6 @@ class Tree:
             import sys
             outfile = sys.stdout
         from time import asctime
-        import common
         outfile.write('<?xml version="1.0"?>\n<!-- generated by wxGlade %s ' \
                       'on %s -->\n\n' % (common.version, asctime()))
         outpath = os.path.expanduser(self.app.output_path.strip())
@@ -295,8 +297,8 @@ class WidgetTree(wx.TreeCtrl, Tree):
         self.cur_widget = None # reference to the selected widget
         Tree.__init__(self, root_node, application)
         image_list = wx.ImageList(21, 21)
-        image_list.Add(wx.Bitmap(os.path.join(common.wxglade_path,
-                                             'icons/application.xpm'),
+        image_list.Add(wx.Bitmap(os.path.join(common.icons_path,
+                                             'application.xpm'),
                                 wx.BITMAP_TYPE_XPM))
         for w in WidgetTree.images:
 ##             WidgetTree.images[w] = image_list.Add(wx.Bitmap(
@@ -345,7 +347,6 @@ class WidgetTree(wx.TreeCtrl, Tree):
         appends child to the list of parent's children
         """
         Tree.add(self, child, parent)
-        import common
         name = child.widget.__class__.__name__
         index = WidgetTree.images.get(name, -1)
         if parent is None: parent = parent.item = self.GetRootItem()
@@ -365,16 +366,13 @@ class WidgetTree(wx.TreeCtrl, Tree):
         if parent.children is None:
             self.add(child, parent, image)
             return
-        import common
         name = child.widget.__class__.__name__
         image_index = WidgetTree.images.get(name, -1)
-        if parent is None: parent = parent.item = self.GetRootItem()
+        if parent is None:
+            parent = parent.item = self.GetRootItem()
 
         index = 0
-        if misc.check_wx_version(2, 5):
-            item, cookie = self.GetFirstChild(parent.item)
-        else:
-            item, cookie = self.GetFirstChild(parent.item, 1)
+        item, cookie = self.GetFirstChild(parent.item)
         while item.IsOk():
             item_pos = self.GetPyData(item).widget.pos
             if pos < item_pos: break
@@ -462,7 +460,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
             self.select_item(node)
             item = node.widget
         if not item.widget or not item.is_visible():
-            import edit_windows
+            #import edit_windows
             #if isinstance(item, edit_windows.TopLevelBase):
             if node.parent is self.root:
                 self._show_menu.SetTitle(item.name)
@@ -474,7 +472,8 @@ class WidgetTree(wx.TreeCtrl, Tree):
             event.m_x, event.m_y = x, y
             item.popup_menu(event)
         except AttributeError:
-            import traceback; traceback.print_exc()
+            import traceback;
+            traceback.print_exc()
 
     def expand(self, node=None, yes=True):
         """\
@@ -680,10 +679,7 @@ class WidgetTree(wx.TreeCtrl, Tree):
             self.select_item(node)
         
     def _get_first_child(self, item):
-        if misc.check_wx_version(2, 5):
-            return self.GetFirstChild(item)
-        else:
-            return self.GetFirstChild(item, 1)
+        return self.GetFirstChild(item)
         
 # end of class WidgetTree
 
