@@ -1,8 +1,8 @@
 # widget_properties.py: classes to handle the various properties of the widgets
 # (name, size, color, etc.)
-# $Id: widget_properties.py,v 1.63 2007/03/31 09:55:14 agriggio Exp $
 # 
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
+#
 # License: MIT (see license.txt)
 # THIS PROGRAM COMES WITH NO WARRANTY
 
@@ -11,19 +11,14 @@
 # property, otherwise on the properties tabs horizontal scrollbars are shown
 _label_initial_width = 5 
 
-#from wxPython.wx import *
-#from wxPython.grid import *
 import wx
 import wx.grid
 from xml.sax.saxutils import escape
 import common, misc
 try:
-    #from wxPython.lib.stattext import *
     import wx.lib.stattext
     wxGenStaticText = wx.lib.stattext.GenStaticText
 except ImportError:
-    # wxGenStaticText has been added in wx 2.3.3, so it may not be available
-    #wxGenStaticText = wxStaticText
     wxGenStaticText = wx.StaticText
 
 def _mangle(label):
@@ -33,8 +28,6 @@ def _mangle(label):
     """
     return misc.wxstr(label.capitalize().replace('_', ' '))
 
-
-import common
 _encode = common._encode_to_xml
 
 
@@ -42,7 +35,7 @@ class Property:
     """\
     A class to handle a single property of a widget.
     """
-    def __init__(self, owner, name, parent, getter=None, setter=None):
+    def __init__(self, owner, name, parent, getter=None, setter=None, label=None):
         # owner: the widget this property belongs to
         # parent: the widget inside which the property is displayed
         """\
@@ -56,6 +49,10 @@ class Property:
         self.getter = getter
         self.setter = setter
         self.name = name
+        if label:
+            self.dispName = label
+        else:
+            self.dispName = name
 
     def on_change_val(self, event, first=[True]):
         """\
@@ -103,7 +100,7 @@ class HiddenProperty(Property):
     """\
     Properties not associated to any control, i.e. not editable by the user.
     """
-    def __init__(self, owner, name, value=None):
+    def __init__(self, owner, name, value=None, label=None):
         try: getter, setter = owner[name]
         except KeyError:
             import traceback; traceback.print_exc()
@@ -114,7 +111,7 @@ class HiddenProperty(Property):
             
         self.panel = None # this is needed to provide an uniform treatment,
                           # but is always None
-        Property.__init__(self, owner, name, None, getter, setter)
+        Property.__init__(self, owner, name, None, getter, setter, label=label)
         if value is not None:
             if callable(value): self.value = value()
             else: self.value = value
@@ -133,32 +130,90 @@ class HiddenProperty(Property):
 
 class _activator:
     """\
-    a utility class which provides a method, toggle_active, to activate or
-    deactivate a Property of a widget
+    a utility class which provides:
+    - a method, toggle_active, to (de)activate a Property of a widget
+    - a method, toggle_blocked, to (un)block enabler of a Property of a widget
+    - a method, prepare_activator, to assign enabler and target with settings  
     """
-    def __init__(self, target=None, enabler=None):
+    def __init__(self, target=None, enabler=None, omitter=None):
         # target: name of the object to Enable/Disable
         # enabler: check box which provides the ability to Enable/Disable the
         #          Property
         self._target = target
         self._enabler = enabler
-        if self._target: self._active = self._target.IsEnabled()
+        if self._enabler: self._blocked = self.is_blocked()
+        else: self._blocked = False
+        if self.is_blocked(): self.toggle_active(False, False)
+        elif self._target: self._active = self.is_active()
         else: self._active = True
+        self.set_omitter( omitter)
 
-    def toggle_active(self, active):
-        self._active = active
+    def toggle_active(self, active=None, refresh=True):
+        # active is not given when refreshing target and enabler
+        if active and self.is_blocked():
+            self._active = False # blocked is always inactive (security measure)
+        elif active is not None:
+            self._active = active
         if not self._target: return
-        self._target.Enable(active)
-        import common
-        try: common.app_tree.app.saved = False
-        except AttributeError: pass # why does this happen on win at startup?
-        try: self._enabler.SetValue(active)
+        try:
+            for target in self._target: 
+                target.Enable(self._active)
+        except TypeError:
+            self._target.Enable(self._active)
+        if self._active and self._omitter:
+            try:
+                value = self.owner.access_functions[self.name][0]()
+                self.owner.access_functions[self.name][1](value)
+            except: pass
+        if refresh:
+            try: common.app_tree.app.saved = False
+            except AttributeError: pass #why does this happen on win at startup?
+        try: self._enabler.SetValue(self._active)
         except AttributeError: pass
 
     def is_active(self):
-        if self._target: return self._target.IsEnabled()
+        if self.is_blocked(): return False
+        if self._target:
+            try:
+                return self._target[0].IsEnabled()
+            except TypeError:
+                return self._target.IsEnabled()
         return self._active
 
+    def toggle_blocked(self, blocked=None):
+        if blocked is not None:
+            self._blocked = blocked
+        if self._enabler:
+            self._enabler.Enable(not self._blocked)
+        if self.is_blocked(): # deactivate blocked
+            self.toggle_active(False, False)
+        elif self._enabler: # refresh activity of target and value of enabler
+            self.toggle_active(refresh=False)
+        elif not getattr(self, 'can_disable', None) and self._omitter:
+            # enable blocked widget (that would be always active) without _enabler
+            if self.owner.get_property_blocking(self._omitter):
+                self.toggle_active(True, False)
+
+    def is_blocked(self):
+        if self._enabler: return not self._enabler.IsEnabled()
+        return self._blocked
+
+    def prepare_activator(self, enabler=None, target=None):
+        if target:
+            self._target = target
+            if enabler:
+                self._enabler = enabler
+        self.toggle_blocked()
+
+    def set_omitter(self, omitter):
+        self._omitter = omitter
+        if omitter:
+            self.owner.set_property_blocking(omitter, self.name)
+
+    def remove_omitter(self):
+        if self._omitter:
+            self.owner.remove_property_blocking(self._omitter, self.name)
+
 # end of class _activator
 
 
@@ -167,14 +222,19 @@ class TextProperty(Property, _activator):
     Properties associated to a text control.
     """
     def __init__(self, owner, name, parent=None, can_disable=False,
-                 enabled=False, readonly=False, multiline=False):
-        Property.__init__(self, owner, name, parent)
+                 enabled=False, readonly=False, multiline=False,
+                 label=None, blocked=False, omitter=None):
+        Property.__init__(self, owner, name, parent, label=label)
         self.val = misc.wxstr(owner[name][0]())
         self.can_disable = can_disable
         self.readonly = readonly
         self.multiline = multiline
-        _activator.__init__(self)
-        if can_disable: self.toggle_active(enabled)
+        _activator.__init__(self, omitter=omitter)
+        if self.name == 'size':
+            pass
+        if can_disable:
+            self.toggle_active(enabled)
+            self.toggle_blocked(blocked)
         self.panel = None
         if parent is not None: self.display(parent)
 
@@ -191,31 +251,26 @@ class TextProperty(Property, _activator):
         if self.multiline: style |= wx.TE_MULTILINE
         val = self.get_value()
         if self.multiline: val = val.replace('\\n', '\n')
-        #label = wxStaticText(self.panel, -1, _mangle(self.name))
-        label = wxGenStaticText(parent, -1, _mangle(self.name),
+        lbl = getattr(self, 'label', None)
+        if lbl is None: lbl = _mangle(self.dispName)
+        #label = wxStaticText(self.panel, -1, _mangle(self.dispName))
+        label = wxGenStaticText(parent, -1, lbl,
                                 size=(_label_initial_width, -1))
-        if self.can_disable:
-            self._enabler = wx.CheckBox(parent, self.id+1, '', size=(1, -1))
         self.text = wx.TextCtrl(parent, self.id, val, style=style, size=(1, -1))
         if hasattr(self, 'tooltip'):
             label.SetToolTip(wx.ToolTip(self.tooltip))
         else:
-            label.SetToolTip(wx.ToolTip(_mangle(self.name)))
+            label.SetToolTip(wx.ToolTip(_mangle(self.dispName)))
+        enabler = None
+        if self.name == 'size':
+            pass
         if self.can_disable:
-            #self._enabler = wxCheckBox(parent, self.id+1, '', size=(1, -1))
-            wx.EVT_CHECKBOX(self._enabler, self.id+1,
+            enabler = wx.CheckBox(parent, self.id+1, '', size=(1, -1))
+            wx.EVT_CHECKBOX(enabler, self.id+1,
                          lambda event: self.toggle_active(event.IsChecked()))
-            self.text.Enable(self.is_active())
-            self._enabler.SetValue(self.is_active())
-            self._target = self.text
+        self.prepare_activator(enabler, self.text)
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(label, 2, wx.ALL|wx.ALIGN_CENTER, 3)
-        #sizer.SetItemMinSize(label, *label.GetBestSize())
-##         try:
-##             sizer.Add(self._enabler, 1, wxALL|wxALIGN_CENTER, 3)
-##             option = 4
-##         except AttributeError:
-##             option = 5
         if getattr(self, '_enabler', None) is not None:
             sizer.Add(self._enabler, 1, wx.ALL|wx.ALIGN_CENTER, 3)
             option = 4
@@ -225,9 +280,6 @@ class TextProperty(Property, _activator):
         if self.multiline:
             h = self.text.GetCharHeight()
             sizer.SetItemMinSize(self.text, -1, h*3)
-##         self.panel.SetAutoLayout(1)
-##         self.panel.SetSizer(sizer)
-##         self.panel.SetSize(sizer.GetMinSize())
         self.panel = sizer
         self.bind_event(self.on_change_val)
         wx.EVT_CHAR(self.text, self.on_char)
@@ -240,8 +292,11 @@ class TextProperty(Property, _activator):
 
     def bind_event(self, function):
         def func_2(event):
+            if self.text.IsBeingDeleted():
+                return
+
             if self.text.IsEnabled():
-                #misc.wxCallAfter(function, event)
+                #wx.CallAfter(function, event)
                 function(event)
             event.Skip()
         wx.EVT_KILL_FOCUS(self.text, func_2)
@@ -270,18 +325,19 @@ class TextProperty(Property, _activator):
 # end of class TextProperty
 
 
-class CheckBoxProperty(Property):
+class CheckBoxProperty(Property, _activator):
     """\
     Properties whose values can be changed by one checkbox.
     """
     def __init__(self, owner, name, parent=None, label=None,
-                 write_always=False):
-        Property.__init__(self, owner, name, parent)
+                 write_always=False, omitter=None):
+        Property.__init__(self, owner, name, parent, label=label)
         self.val = int(owner[name][0]())
-        if label is None: label = _mangle(name)
+        if label is None or label == name: label = _mangle(name)
         self.label = label
         self.panel = None
         self.write_always = write_always
+        _activator.__init__(self, omitter=omitter)
         if parent is not None: self.display(parent)
 
     def display(self, parent):
@@ -293,10 +349,15 @@ class CheckBoxProperty(Property):
         #self.panel = wxPanel(parent, -1)
         self.cb = wx.CheckBox(parent, self.id, '')
         self.cb.SetValue(self.val)
-        label = wx.StaticText(parent, -1, self.label)
+        label = wxGenStaticText(parent, -1, self.label)
+        if hasattr(self, 'tooltip'):
+            label.SetToolTip(wx.ToolTip(self.tooltip))
+        else:
+            label.SetToolTip(wx.ToolTip(self.label))
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(label, 5, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 3)
-        sizer.Add(self.cb, 2, wx.ALIGN_CENTER|wx.ALL, 3)
+        sizer.Add(self.cb, 0, wx.ALIGN_CENTER|wx.ALL, 3)
+        self.prepare_activator(target=self.cb)
 ##         self.panel.SetAutoLayout(True)
 ##         self.panel.SetSizer(sizer)
 ##         self.panel.SetSize(sizer.GetMinSize())
@@ -327,12 +388,12 @@ class CheckBoxProperty(Property):
 # end of class CheckBoxProperty
 
 
-class CheckListProperty(Property):
+class CheckListProperty(Property, _activator):
     """\
     Properties whose values can be changed by a list of checkboxes.
     """
     def __init__(self, owner, name, parent=None, labels=None, writer=None,
-                 tooltips=None):
+                 tooltips=None, omitter=None):
         """
         @type labels: list of strings
         @param labels: list of names of the labels of the checkboxes; a
@@ -343,7 +404,7 @@ class CheckListProperty(Property):
         @param tooltips: a list of strings to be displayed as the tool-tips for
         the properties
         """ #" # ALB - fix for emacs's syntax highlight, don't remove please!
-        Property.__init__(self, owner, name, parent)
+        Property.__init__(self, owner, name, parent, label=None)
         self.values = owner[name][0]()
         self.labels = labels
         self.tooltips = tooltips
@@ -351,6 +412,7 @@ class CheckListProperty(Property):
         # for this property
         self.writer = writer
         self.panel = None
+        _activator.__init__(self, omitter=omitter)
         if parent is not None: self.display(parent)
 
     def display(self, parent):
@@ -371,8 +433,9 @@ class CheckListProperty(Property):
             if self.labels[i].startswith('#section#'):
                 if tmp != tmp_sizer:
                     tmp_sizer.Add(tmp, 1, wx.ALL|wx.EXPAND, 5)
-                lbl = _mangle(self.labels[i].replace('#section#', ''))
-                tmp = wx.StaticBoxSizer(wx.StaticBox(parent, -1, lbl),
+                lbl = _mangle(self.labels[i].replace(u'#section#', ''))
+                s = wx.FULL_REPAINT_ON_RESIZE
+                tmp = wx.StaticBoxSizer(wx.StaticBox(parent, -1, lbl, style=s),
                                        wx.VERTICAL)
             else:
                 c = wx.CheckBox(parent, self.id+j, self.labels[i])
@@ -381,16 +444,17 @@ class CheckListProperty(Property):
                 j += 1
             i += 1
         if tmp != tmp_sizer:
-            tmp_sizer.Add(tmp, 1, wx.ALL|wx.EXPAND, 5)
+            tmp_sizer.Add(tmp, 0, wx.ALL|wx.EXPAND, 5)
 
+        self.prepare_activator(target=self.choices)
         for i in range(len(self.values)):
             self.choices[i].SetValue(self.values[i])
-	    
-	#Set the tool-tips for the properties
-	if self.tooltips is not None:
-		for i in range(len(self.tooltips)):
-			if i >= len(self.choices): break
-			self.choices[i].SetToolTip(wx.ToolTip(self.tooltips[i]))
+            
+        #Set the tool-tips for the properties
+        if self.tooltips is not None:
+                for i in range(len(self.tooltips)):
+                        if i >= len(self.choices): break
+                        self.choices[i].SetToolTip(wx.ToolTip(self.tooltips[i]))
                   
 ##         self.panel.SetSizer(tmp_sizer)
 ##         self.panel.SetSize(tmp_sizer.GetMinSize())
@@ -446,19 +510,25 @@ class SpinProperty(Property, _activator):
     Properties associated to a spin control.
     """
     def __init__(self, owner, name, parent=None, can_disable=False,
-                 r=None, enabled=False):
+                 r=None, enabled=False, immediate=False, label=None,
+                 blocked=False, omitter=None):
         # r = range of the spin (min, max)
-        Property.__init__(self, owner, name, parent)
+        Property.__init__(self, owner, name, parent, label=label)
         self.can_disable = can_disable
-        _activator.__init__(self)
-        if can_disable: self.toggle_active(enabled)
+        self.immediate = immediate # if true, changes to this property have an
+                                   # immediate effect (instead of waiting the
+                                   # focus change...)
+        _activator.__init__(self, omitter=omitter)
+        if can_disable:
+            self.toggle_active(enabled)
+            self.toggle_blocked(blocked)
         if r is not None:
             self.val_range = (r[0], max(r[0], r[1]))
         else:
             self.val_range = None
         self.panel = None
-        if parent is not None: self.display(parent)
         self.val = owner[name][0]()
+        if parent is not None: self.display(parent)
 
     def display(self, parent):
         """\
@@ -467,24 +537,25 @@ class SpinProperty(Property, _activator):
         """
         self.id = wx.NewId()
         if self.val_range is None: self.val_range = (0, 1000)
-        label = wxGenStaticText(parent, -1, _mangle(self.name),
+        lbl = getattr(self, 'label', None)
+        if lbl is None: lbl = _mangle(self.dispName)
+        label = wxGenStaticText(parent, -1, lbl,
                                 size=(_label_initial_width, -1))
-        label.SetToolTip(wx.ToolTip(_mangle(self.name)))
-        if self.can_disable:
-            self._enabler = wx.CheckBox(parent, self.id+1, '', size=(1, -1))
+        tip = getattr(self, 'tooltip', None)
+        if tip is None: tip = lbl
+        label.SetToolTip(wx.ToolTip(tip))
         self.spin = wx.SpinCtrl(parent, self.id, min=self.val_range[0],
                                max=self.val_range[1])
         val = int(self.owner[self.name][0]())
         if not val:
             self.spin.SetValue(1) # needed for GTK to display a '0'
         self.spin.SetValue(val) #int(self.owner[self.name][0]()))
+        enabler = None
         if self.can_disable:
-            #self._enabler = wxCheckBox(parent, self.id+1, '', size=(1, -1))
-            wx.EVT_CHECKBOX(self._enabler, self.id+1,
+            enabler = wx.CheckBox(parent, self.id+1, '', size=(1, -1))
+            wx.EVT_CHECKBOX(enabler, self.id+1,
                          lambda event: self.toggle_active(event.IsChecked()))
-            self.spin.Enable(self.is_active())
-            self._enabler.SetValue(self.is_active())
-            self._target = self.spin
+        self.prepare_activator(enabler, self.spin)
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(label, 2, wx.ALL|wx.ALIGN_CENTER, 3)
         if getattr(self, '_enabler', None) is not None:
@@ -498,14 +569,14 @@ class SpinProperty(Property, _activator):
 
     def bind_event(self, function):
         def func_2(event):
+            if self.spin.IsBeingDeleted():
+                return
+            
             if self.is_active():
-##                 if wxPlatform != '__WXMSW__':
-##                     misc.wxCallAfter(function, event)
-##                 else:
                 function(event)
             event.Skip()
         wx.EVT_KILL_FOCUS(self.spin, func_2)
-        if wx.Platform == '__WXMAC__':
+        if wx.Platform == '__WXMAC__' or self.immediate:
             wx.EVT_TEXT(self.spin, self.spin.GetId(), func_2)
             wx.EVT_SPINCTRL(self.spin, self.spin.GetId(), func_2)
 
@@ -536,13 +607,15 @@ class DialogProperty(Property, _activator):
     get_value method.
     """
     def __init__(self, owner, name, parent, dialog, can_disable=False,
-                 enabled=False):
-        Property.__init__(self, owner, name, parent)
+                 enabled=False, label=None, blocked=False, omitter=None):
+        Property.__init__(self, owner, name, parent, label=label)
         self.dialog = dialog
         self.panel = None
         self.can_disable = can_disable
-        _activator.__init__(self)
-        if can_disable: self.toggle_active(enabled)
+        _activator.__init__(self, omitter=omitter)
+        if can_disable:
+            self.toggle_active(enabled)
+            self.toggle_blocked(blocked)
         if parent is not None: self.display(parent)
         self.val = "%s" % owner[name][0]()
 
@@ -553,22 +626,20 @@ class DialogProperty(Property, _activator):
         """
         self.id = wx.NewId()
         val = misc.wxstr(self.owner[self.name][0]())
-        label = wxGenStaticText(parent, -1, _mangle(self.name),
+        label = wxGenStaticText(parent, -1, _mangle(self.dispName),
                                 size=(_label_initial_width, -1))
-        label.SetToolTip(wx.ToolTip(_mangle(self.name)))
-        if self.can_disable:
-            self._enabler = wx.CheckBox(parent, self.id+1, '', size=(1, -1))
+        label.SetToolTip(wx.ToolTip(_mangle(self.dispName)))
         self.text = wx.TextCtrl(parent, self.id, val, size=(1, -1))
         self.btn = wx.Button(parent, self.id+1, " ... ",
                             size=(_label_initial_width, -1))
+        enabler = None
         if self.can_disable:
-            #self._enabler = wxCheckBox(parent, self.id+1, '', size=(1, -1))
-            wx.EVT_CHECKBOX(self._enabler, self.id+1,
+            enabler = wx.CheckBox(parent, self.id+1, '', size=(1, -1))
+            wx.EVT_CHECKBOX(enabler, self.id+1,
                          lambda event: self.toggle_active(event.IsChecked()))
-            self.text.Enable(self.is_active())
+        self.prepare_activator(enabler, self.text)
+        if self.can_disable:
             self.btn.Enable(self.is_active())
-            self._enabler.SetValue(self.is_active())
-            self._target = self.text
         wx.EVT_BUTTON(self.btn, self.id+1, self.display_dialog)
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(label, 2, wx.ALL|wx.ALIGN_CENTER, 3)
@@ -596,7 +667,12 @@ class DialogProperty(Property, _activator):
         self.text.ProcessEvent(wx.FocusEvent(wx.wxEVT_KILL_FOCUS, self.id))
 
     def bind_event(self, function):
-        wx.EVT_KILL_FOCUS(self.text, function)
+        def func_2(event):
+            if self.text.IsBeingDeleted():
+                return
+
+            function(event)
+        wx.EVT_KILL_FOCUS(self.text, func_2)
 
     def get_value(self):
         try: return self.text.GetValue()
@@ -611,10 +687,10 @@ class DialogProperty(Property, _activator):
         if self.is_active():
             Property.write(self, dest_file, tabs)
 
-    def toggle_active(self, active):
-        _activator.toggle_active(self, active)
+    def toggle_active(self, active=None, refresh=True):
+        _activator.toggle_active(self, active, refresh)
         try:
-            self.btn.Enable(active)
+            self.btn.Enable(self.is_active())
         except AttributeError:
             pass
 
@@ -625,7 +701,7 @@ class FileDialogProperty(DialogProperty):
     dialog = [None]
 
     class FileDialog:
-        def __init__(self, parent, message, wildcard, style):
+        def __init__(self, parent, message, wildcard, style, label=None):
             self.parent = parent
             self.message = message
             self.wildcard = wildcard
@@ -644,7 +720,8 @@ class FileDialogProperty(DialogProperty):
     # end of class FileDialog
     
     def __init__(self, owner, name, parent=None, wildcard=_("All Files|*"),
-                 message=_("Choose a file"), can_disable=True, style=0):
+                 message=_("Choose a file"), can_disable=True, style=0,
+                 label=None, blocked=False, omitter=None):
         if not self.dialog[0]:
 ##             self.dialog[0] = wxFileDialog(parent, message,
 ##                                           wildcard=wildcard, style=style)
@@ -652,7 +729,8 @@ class FileDialogProperty(DialogProperty):
             self.dialog[0] = self.FileDialog(
                 parent, message, wildcard, style)
         DialogProperty.__init__(self, owner, name, parent, self.dialog[0],
-                                can_disable)
+                                can_disable, label=label, blocked=blocked,
+                                omitter=omitter)
 
 # end of class FileDialogProperty
 
@@ -698,26 +776,29 @@ class ColorDialogProperty(DialogProperty):
     colors_to_str = _reverse_dict(str_to_colors)
 
     dialog = [None]
-    def __init__(self, owner, name, parent=None, can_disable=True):
+    def __init__(self, owner, name, parent=None, can_disable=True,
+                 label=None, blocked=False, omitter=None):
         if not self.dialog[0]:
             from color_dialog import wxGladeColorDialog
             self.dialog[0] = wxGladeColorDialog(self.str_to_colors)
         DialogProperty.__init__(self, owner, name, parent, self.dialog[0],
-                                can_disable)
+                                can_disable, label=label, blocked=blocked,
+                                omitter=omitter)
 
     def display_dialog(self, event):
         self.dialog.set_value(self.get_value())
         DialogProperty.display_dialog(self, event)
 
-    def toggle_active(self, active):
-        DialogProperty.toggle_active(self, active)
-        if not active:
+    def toggle_active(self, active=None, refresh=True):
+        DialogProperty.toggle_active(self, active, refresh)
+        if not self.is_active():
             # restore the original value if toggled off
             color = self.owner._original.get(self.name, None)
             if color is not None and self.owner.widget is not None:
                 which = 'Set%sColour' % self.name.capitalize()
                 func = getattr(self.owner.widget, which, lambda c: None)
                 func(color)
+                self.owner.widget.Refresh()
         else:
             # restore the saved value
             getval, setval = self.owner[self.name]
@@ -737,18 +818,19 @@ class FontDialogProperty(DialogProperty):
     font_weights_to = {'normal': wx.NORMAL, 'light': wx.LIGHT, 'bold': wx.BOLD }
     font_weights_from = _reverse_dict(font_weights_to)
     
-    if misc.check_wx_version(2, 3, 3):
-        font_families_to['teletype'] = wx.TELETYPE 
-        font_families_from[wx.TELETYPE] = 'teletype' 
+    font_families_to['teletype'] = wx.TELETYPE 
+    font_families_from[wx.TELETYPE] = 'teletype' 
 
     dialog = [None]
 
-    def __init__(self, owner, name, parent=None, can_disable=True):
+    def __init__(self, owner, name, parent=None, can_disable=True,
+                 label=None, blocked=False, omitter=None):
         if not self.dialog[0]:
             import font_dialog
             self.dialog[0] = font_dialog.wxGladeFontDialog(parent, -1, "")
         DialogProperty.__init__(self, owner, name, parent, self.dialog[0],
-                                can_disable)
+                                can_disable, label=label, blocked=blocked,
+                                omitter=omitter)
 
     def display_dialog(self, event):
         try: props = eval(self.get_value())
@@ -781,9 +863,9 @@ class FontDialogProperty(DialogProperty):
             fwrite('%s<face>%s</face>\n' % (tstr, escape(props[5])))
             fwrite('    ' * tabs + '</%s>\n' % self.name)
 
-    def toggle_active(self, active):
-        DialogProperty.toggle_active(self, active)
-        if not active:
+    def toggle_active(self, active=None, refresh=True):
+        DialogProperty.toggle_active(self, active, refresh)
+        if not self.is_active():
             # restore the original value if toggled off
             font = self.owner._original['font']
             if font is not None and self.owner.widget is not None:
@@ -802,19 +884,23 @@ class RadioProperty(Property, _activator):
     properties controlled by a series of radio buttons.
     """
     def __init__(self, owner, name, parent, choices, can_disable=False,
-                 enabled=False, columns=1, label=None):
-        Property.__init__(self, owner, name, parent)
+                 enabled=False, columns=1, label=None, tooltips=None,
+                 blocked=False, omitter=None):
+        Property.__init__(self, owner, name, parent, label=label)
         self.can_disable = can_disable
-        _activator.__init__(self)
-        if can_disable: self.toggle_active(enabled)
+        _activator.__init__(self, omitter=omitter)
+        if can_disable:
+            self.toggle_active(enabled)
+            self.toggle_blocked(blocked)
         self.choices = choices
         self.columns = columns
         self.panel = None
         self.label = label
+        self.tooltips = tooltips
+        self.val = owner[name][0]()
         if label is None:
             self.label = _mangle(name)
         if parent is not None: self.display(parent)
-        self.val = owner[name][0]()
 
     def display(self, parent):
         """\
@@ -822,27 +908,32 @@ class RadioProperty(Property, _activator):
         interactively
         """
         self.id = wx.NewId()
-        style = wx.RA_SPECIFY_COLS|wx.NO_BORDER
+        style = wx.RA_SPECIFY_COLS|wx.NO_BORDER|wx.CLIP_CHILDREN
         if not self.can_disable: 
             szr = wx.BoxSizer(wx.HORIZONTAL)
             style=wx.RA_SPECIFY_COLS
         else: 
             szr = wx.StaticBoxSizer(wx.StaticBox(parent, -1, self.label),
-                                   wx.HORIZONTAL)
+                                    wx.HORIZONTAL)
         self.options = wx.RadioBox(parent, self.id, self.label,
-                                  choices=self.choices,
-                                  majorDimension=self.columns,
-                                  style=style)
+                                   choices=self.choices,
+                                   majorDimension=self.columns,
+                                   style=style)
+        #Set the tool-tips for the properties
+        if self.tooltips is not None:
+            for i in range(len(self.tooltips)):
+                if i >= len(self.choices): break
+                self.options.SetItemToolTip(i, self.tooltips[i])
         try: self.options.SetSelection(int(self.val))
         except: pass
+        enabler = None
         if self.can_disable:
-            self._enabler = wx.CheckBox(parent, self.id+1, "")
-            szr.Add(self._enabler)
-            wx.EVT_CHECKBOX(self._enabler, self.id+1,
+            enabler = wx.CheckBox(parent, self.id+1, "")
+            szr.Add(enabler)
+            wx.EVT_CHECKBOX(enabler, self.id+1,
                          lambda e: self.toggle_active(e.IsChecked()))
-            self.options.Enable(self.is_active())
             self.options.SetLabel("")
-            self._enabler.SetValue(self.is_active())
+        self.prepare_activator(enabler, self.options)
         szr.Add(self.options, 1, wx.EXPAND)
         self.panel = szr
         self.bind_event(self.on_change_val)
@@ -885,29 +976,34 @@ class RadioProperty(Property, _activator):
 # end of class RadioProperty
 
 
-class GridProperty(Property): #wxPanel, Property):
+class GridProperty(Property, _activator): 
     """\
     Property whose values are modified through a wxGrid table.
     """
-    STRING, INT, FLOAT = 0, 1, 2
+    STRING, INT, FLOAT, BOOL = 0, 1, 2, 3
     col_format = [lambda g, c: None,
                   lambda g, c: g.SetColFormatNumber(c),
-                  lambda g, c: g.SetColFormatFloat(c)]
+                  lambda g, c: g.SetColFormatFloat(c),
+                  lambda g, c: g.SetColFormatBool(c)]
     def __init__(self, owner, name, parent, cols, rows=1, can_add=True,
-                 can_remove=True, can_insert=True):
+                 can_remove=True, can_insert=True, label=None, omitter=None,
+                 col_sizes=[]):
         # cols: list of 2-tuples with these fields:
         #     - label for the column
         #     - type: GridProperty.STRING, GridProperty.INT, GridProperty.FLOAT
         # rows: number of rows
-        Property.__init__(self, owner, name, parent)
+        # col_sizes: list of column widths
+        Property.__init__(self, owner, name, parent, label=label)
         self.val = owner[name][0]()
         self.set_value(self.val)
         self.rows, self.cols = rows, cols
         self.can_add = can_add
         self.can_remove = can_remove
         self.can_insert = can_insert
+        self.col_sizes = col_sizes
         self.panel = None
         self.cur_row = 0
+        _activator.__init__(self, omitter=omitter)
         if parent is not None: self.display(parent)
 
     def display(self, parent):
@@ -915,29 +1011,34 @@ class GridProperty(Property): #wxPanel, Property):
         Actually builds the grid to set the value of the property
         interactively
         """
-        self.panel = wx.Panel(parent, -1) # why if the grid is not on this panel
-                                         # it is not displayed???
-        sizer = wx.StaticBoxSizer(wx.StaticBox(self.panel, -1,
-                                             _mangle(self.name)), wx.VERTICAL)
+        children = []
+        self.panel = wx.Panel(parent, -1) # why if the grid is not on this
+                                          # panel it is not displayed???
+        label = getattr(self, 'label', _mangle(self.dispName))
+        sizer = wx.StaticBoxSizer(wx.StaticBox(self.panel, -1, label),
+                                  wx.VERTICAL)
         self.btn_id = wx.NewId()
         self.btn = wx.Button(self.panel, self.btn_id, _("  Apply  "),
-                            style=wx.BU_EXACTFIT)
+                             style=wx.BU_EXACTFIT)
+        children.append(self.btn)
         if self.can_add:
             self.add_btn = wx.Button(self.panel, self.btn_id+1, _("  Add  "),
-                                    style=wx.BU_EXACTFIT)
+                                     style=wx.BU_EXACTFIT)
+            children.append(self.add_btn)
         if self.can_insert:
-            self.insert_btn = wx.Button(self.panel, self.btn_id+3, _("  Insert  "),
-                                       style=wx.BU_EXACTFIT)
+            self.insert_btn = wx.Button(self.panel, self.btn_id+3,
+                                        _("  Insert  "),
+                                        style=wx.BU_EXACTFIT)
+            children.append(self.insert_btn)
         if self.can_remove:
-            self.remove_btn = wx.Button(self.panel, self.btn_id+2, _("  Remove  "),
-                                       style=wx.BU_EXACTFIT)
+            self.remove_btn = wx.Button(self.panel, self.btn_id+2,
+                                        _("  Remove  "),
+                                        style=wx.BU_EXACTFIT)
+            children.append(self.remove_btn)
         self.grid = wx.grid.Grid(self.panel, -1)
         self.grid.CreateGrid(self.rows, len(self.cols))
-        if misc.check_wx_version(2, 3, 3):
-            self.grid.SetMargins(0, 0)
-        else:
-            # wx 2.3.2 seems to have some problems with grid scrollbars...
-            self.grid.SetMargins(0, self.grid.GetDefaultRowSize())
+        children.append(self.grid)
+        self.grid.SetMargins(0, 0)
 
         for i in range(len(self.cols)):
             self.grid.SetColLabelValue(i, misc.wxstr(self.cols[i][0]))
@@ -946,11 +1047,12 @@ class GridProperty(Property): #wxPanel, Property):
         self.cols = len(self.cols)
         self.grid.SetRowLabelSize(0)
         self.grid.SetColLabelSize(20)
+        if self.col_sizes:
+            self.set_col_sizes(self.col_sizes)
 
         self.btn_sizer = wx.BoxSizer(wx.HORIZONTAL)
         _w = self.btn.GetTextExtent(self.btn.GetLabel())[0]
-        if misc.check_wx_version(2, 5, 2): extra_flag = wx.FIXED_MINSIZE
-        else: extra_flag = 0
+        extra_flag = wx.FIXED_MINSIZE
         #self.btn.SetSize((_w, -1))
         self.btn_sizer.Add(self.btn, 0, extra_flag)
         if self.can_add:
@@ -975,6 +1077,7 @@ class GridProperty(Property): #wxPanel, Property):
         self.panel.SetSizer(sizer)
         self.panel.SetSize(sizer.GetMinSize())
 
+        self.prepare_activator(target=children)
         wx.grid.EVT_GRID_SELECT_CELL(self.grid, self.on_select_cell)
         self.bind_event(self.on_change_val)
         self.set_value(self.val)
@@ -1083,9 +1186,9 @@ class ComboBoxProperty(Property, _activator):
     Properties whose values can be changed with a combobox.
     """
     def __init__(self, owner, name, choices, parent=None, label=None,
-                 can_disable=False, enabled=False,
-                 write_always=False):
-        Property.__init__(self, owner, name, parent)
+                 can_disable=False, enabled=False, write_always=False,
+                 blocked=False, omitter=None):
+        Property.__init__(self, owner, name, parent, label=label)
         self.val = misc.wxstr(owner[name][0]())
         if label is None: label = _mangle(name)
         self.label = label
@@ -1093,8 +1196,10 @@ class ComboBoxProperty(Property, _activator):
         self.write_always = write_always
         self.choices = choices
         self.can_disable = can_disable
-        _activator.__init__(self)
-        if can_disable: self.toggle_active(enabled)
+        _activator.__init__(self, omitter=omitter)
+        if can_disable: 
+            self.toggle_active(enabled)
+            self.toggle_blocked(blocked)
         if parent is not None: self.display(parent)
 
     def display(self, parent):
@@ -1103,19 +1208,20 @@ class ComboBoxProperty(Property, _activator):
         interactively
         """
         self.id = wx.NewId()
-        #self.panel = wxPanel(parent, -1)
         self.cb = wx.ComboBox(parent, self.id, choices=self.choices,
                               style=wx.CB_DROPDOWN|wx.CB_READONLY)
         self.cb.SetValue(self.val)
-        if self.can_disable:
-            self._enabler = wx.CheckBox(parent, self.id+1, '', size=(1, -1))
         label = wx.StaticText(parent, -1, self.label)
+        if hasattr(self, 'tooltip'):
+            label.SetToolTip(wx.ToolTip(self.tooltip))
+        else:
+            label.SetToolTip(wx.ToolTip(_mangle(self.dispName)))
+        enabler = None
         if self.can_disable:
-            wx.EVT_CHECKBOX(self._enabler, self.id+1,
+            enabler = wx.CheckBox(parent, self.id+1, '', size=(1, -1))
+            wx.EVT_CHECKBOX(enabler, self.id+1,
                          lambda event: self.toggle_active(event.IsChecked()))
-            self.cb.Enable(self.is_active())
-            self._enabler.SetValue(self.is_active())
-            self._target = self.cb
+        self.prepare_activator(enabler, self.cb)
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(label, 2, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 3)
         if getattr(self, '_enabler', None) is not None:
@@ -1124,9 +1230,6 @@ class ComboBoxProperty(Property, _activator):
         else:
             option = 5
         sizer.Add(self.cb, option, wx.ALIGN_CENTER|wx.ALL, 3)
-##         self.panel.SetAutoLayout(True)
-##         self.panel.SetSizer(sizer)
-##         self.panel.SetSize(sizer.GetMinSize())
         self.panel = sizer
         self.bind_event(self.on_change_val)
         
@@ -1143,7 +1246,7 @@ class ComboBoxProperty(Property, _activator):
         except AttributeError: pass
 
     def write(self, outfile, tabs):
-        if self.write_always or self.get_value():
+        if self.write_always or (self.is_active() and self.get_value()):
             if self.getter: value = misc.wxstr(self.getter())
             else: value = misc.wxstr(self.owner[self.name][0]())
             if value != 'None':
