@@ -1,6 +1,6 @@
 # xml_parse.py: parsers used to load an app and to generate the code
 # from an xml file.
-# $Id: xml_parse.py,v 1.45 2007/03/27 07:02:07 agriggio Exp $
+# $Id: xml_parse.py,v 1.46 2007/08/07 12:16:44 agriggio Exp $
 #
 # Copyright (c) 2002-2007 Alberto Griggio <agriggio@users.sourceforge.net>
 # License: MIT (see license.txt)
@@ -28,7 +28,6 @@ import xml.sax.expatreader
 
 
 if common.use_gui:
-    #from wxPython import wx
     import wx
 
 class XmlParsingError(SAXException):
@@ -39,7 +38,7 @@ class XmlParsingError(SAXException):
     def __init__(self, msg):
         if self.locator:
             l = self.locator
-            msg += ' (line: %s, column:  %s)' % (l.getLineNumber(),
+            msg += ' _((line: %s, column:  %s))' % (l.getLineNumber(),
                                                  l.getColumnNumber())
         SAXException.__init__(self, msg)
 
@@ -70,7 +69,7 @@ class XmlParser(ContentHandler):
         self.parser.parse(source)
 
     def parse_string(self, source):
-        from cStringIO import StringIO
+        from StringIO import StringIO
         source = StringIO(source)
         self.parser.parse(source)
         source.close()
@@ -142,11 +141,11 @@ class XmlWidgetBuilder(XmlParser):
             if use_gettext:
                 app.use_gettext = True
                 app.use_gettext_prop.set_value(True)
-		
+                
             try: is_template = int(attrs["is_template"])
             except (KeyError, ValueError): is_template = False
             app.is_template = is_template
-	    
+            
             try: overwrite = int(attrs['overwrite'])
             except (KeyError, ValueError): overwrite = False
             if overwrite:
@@ -160,6 +159,31 @@ class XmlWidgetBuilder(XmlParser):
             app.set_use_old_namespace(not use_new_namespace)
             app.use_old_namespace_prop.set_value(not use_new_namespace)
             # ALB 2004-12-05
+            
+            indent_symbol = attrs.get("indent_symbol")
+            if indent_symbol == 'space':
+                app.indent_mode = 1
+            elif indent_symbol == 'tab':
+                app.indent_mode = 0
+            app.indent_mode_prop.set_value (app.indent_mode)
+            
+            indent = attrs.get("indent_amount")
+            if indent:
+                try: indent_amount = int(indent)
+                except (KeyError, ValueError): pass
+                else: 
+                    app.indent_amount = indent_amount
+                    app.indent_amount_prop.set_value (indent_amount)
+            
+            source_extension = attrs.get("source_extension")
+            if source_extension and source_extension[0] == '.':
+                app.source_ext = source_extension[1:]
+                app.source_ext_prop.set_value (source_extension[1:])
+            header_extension = attrs.get("header_extension")
+            if header_extension and header_extension[0] == '.':
+                app.header_ext = header_extension[1:]
+                app.header_ext_prop.set_value (header_extension[1:])
+
             try:
                 for_version = attrs['for_version']
                 app.for_version = for_version
@@ -168,7 +192,7 @@ class XmlWidgetBuilder(XmlParser):
                 pass
             return
         if not self._appl_started:
-            raise XmlParsingError("the root of the tree must be <application>")
+            raise XmlParsingError(_("the root of the tree must be <application>"))
         if name == 'object':
             # create the object and push it on the appropriate stacks
             XmlWidgetObject(attrs, self)
@@ -235,8 +259,8 @@ class XmlWidgetBuilder(XmlParser):
         if not data or data.isspace():
             return
         if self._curr_prop is None:
-            raise XmlParsingError("character data can be present only " \
-                                  "inside properties")
+            raise XmlParsingError(_("character data can be present only " \
+                                  "inside properties"))
         self._curr_prop_val.append(data)
 
 # end of class XmlWidgetBuilder
@@ -344,7 +368,7 @@ class ClipboardXmlWidgetBuilder(XmlWidgetBuilder):
                 try:
                     self.top_obj = self.top().obj
                 except AttributeError:
-                    print 'Exception! obj: %s' % self.top_obj
+                    print _('Exception! obj: %s') % self.top_obj
                     traceback.print_exc()
             self.depth_level += 1
 
@@ -360,7 +384,7 @@ class ClipboardXmlWidgetBuilder(XmlWidgetBuilder):
                     self.top_obj.show_properties()
                     common.app_tree.select_item(self.top_obj.node)
                 except AttributeError:
-                    print 'Exception! obj: %s' % self.top_obj
+                    print _('Exception! obj: %s') % self.top_obj
                     traceback.print_exc()
         XmlWidgetBuilder.endElement(self, name)
 
@@ -382,8 +406,8 @@ class XmlWidgetObject:
             base = attrs.get('base', None)
             self.klass = attrs['class']
         except KeyError:
-            raise XmlParsingError("'object' items must have a 'class' " \
-                                  "attribute")
+            raise XmlParsingError(_("'object' items must have a 'class' " \
+                                  "attribute"))
                   
         if base is not None:
             # if base is not None, the object is a widget (or sizer), and not a
@@ -439,7 +463,7 @@ class XmlWidgetObject:
         elif self.klass == 'sizerslot':
             sizer = self.parser._sizers.top().obj
             assert sizer is not None, \
-                   "malformed wxg file: slots can only be inside sizers!"
+                   _("malformed wxg file: slots can only be inside sizers!")
             sizer.add_slot()
             self.parser._sizer_item.push(self)
                         
@@ -471,8 +495,8 @@ class XmlWidgetObject:
             # unknown property for this object
             # issue a warning and ignore the property
             import sys
-            print >> sys.stderr, "Warning: property '%s' not supported by " \
-                  "this object ('%s') " % (name, self.obj)
+            print >> sys.stderr, _("Warning: property '%s' not supported by " \
+                  "this object ('%s') ") % (name, self.obj)
 
 #end of class XmlWidgetObject
 
@@ -480,7 +504,7 @@ class XmlWidgetObject:
 class CodeWriter(XmlParser):
     """parser used to produce the source from a given xml file"""
     def __init__(self, writer, input, from_string=False, out_path=None,
-                 preview=False):
+                 preview=False, class_names=None):
         # writer: object that actually writes the code
         XmlParser.__init__(self)
         self._toplevels = Stack() # toplevel objects, i.e. instances of a
@@ -494,6 +518,11 @@ class CodeWriter(XmlParser):
 
         self.preview = preview # if True, we are generating the code for the
                                # preview
+                               
+        # used in the CustomWidget preview code, to generate better previews
+        # (see widgets/custom_widget/codegen.py)
+        self.class_names = class_names
+        if self.class_names is None: self.class_names = set()
 
         if from_string: self.parse_string(input)
         else:
@@ -510,7 +539,10 @@ class CodeWriter(XmlParser):
             encoding = self.app_attrs['encoding']
             unicode('a', encoding)
         except (KeyError, LookupError):
-            encoding = 'ISO-8859-1'
+            if name == 'application':
+                encoding = str(attrs_impl.get('encoding', 'ISO-8859-1'))
+            else:
+                encoding = 'ISO-8859-1'
         # turn all the attribute values from unicode to str objects
         for attr, val in attrs_impl.items():
             attrs[attr] = common._encode_from_xml(val, encoding)
@@ -526,26 +558,26 @@ class CodeWriter(XmlParser):
             if self.out_path is None:
                 try: self.out_path = attrs['path']
                 except KeyError:
-                    raise XmlParsingError("'path' attribute missing: could "
-                                          "not generate code")
+                    raise XmlParsingError(_("'path' attribute missing: could "
+                                          "not generate code"))
             else: attrs['path'] = self.out_path
 
             # ALB 2004-11-01: check if the values of
             # use_multiple_files and out_path agree
             if use_multiple_files:
                 if not os.path.isdir(self.out_path):
-                    raise IOError("Output path must be an existing directory"
-                                  " when generating multiple files")
+                    raise IOError(_("Output path must be an existing directory"
+                                  " when generating multiple files"))
             else:
                 if os.path.isdir(self.out_path):
-                    raise IOError("Output path can't be a directory when "
-                                  "generating a single file")
+                    raise IOError(_("Output path can't be a directory when "
+                                  "generating a single file"))
             
             # initialize the writer
             self.code_writer.initialize(attrs)
             return
         if not self._appl_started:
-            raise XmlParsingError("the root of the tree must be <application>")
+            raise XmlParsingError(_("the root of the tree must be <application>"))
         if name == 'object':
             # create the CodeObject which stores info about the current widget
             CodeObject(attrs, self, preview=self.preview)
@@ -631,8 +663,8 @@ class CodeWriter(XmlParser):
     def characters(self, data):
         if not data or data.isspace(): return
         if self._curr_prop is None:
-            raise XmlParsingError("character data can only appear inside " \
-                                  "properties")
+            raise XmlParsingError(_("character data can only appear inside " \
+                                  "properties"))
         self._curr_prop_val.append(data)
 
 # end of class CodeWriter
@@ -641,17 +673,26 @@ class CodeWriter(XmlParser):
 class CodeObject:
     """\
     A class to store information needed to generate the code for a given object
+
+    @ivar is_container: If True, the widget is a container (frame, dialog,
+                        panel, ...)
+    @type is_container: Boolean
+
+    @ivar is_toplevel: If True, the object is a toplevel one: for window
+                       objects, this means that they are instances of a
+                       custom class, for sizers, that they are at the top
+                       of the hierarchy.
+    @type is_toplevel: Boolean
+
+    @ivar properties: Properties of the widget sizer
+    @type properties: Dictionary
     """
     def __init__(self, attrs, parser, preview=False):
         self.parser = parser
         self.in_windows = self.in_sizers = False
-        self.is_toplevel = False # if True, the object is a toplevel one:
-                             # for window objects, this means that they are
-                             # instances of a custom class, for sizers, that
-                             # they are at the top of the hierarchy
-        self.is_container = False # if True, the widget is a container
-                                  # (frame, dialog, panel, ...)
-        self.properties = {} # properties of the widget/sizer
+        self.is_toplevel = False
+        self.is_container = False
+        self.properties = {}
         # prop_handlers is a stack of custom handler functions to set
         # properties of this object
         self.prop_handlers = Stack()
@@ -660,8 +701,8 @@ class CodeObject:
             base = attrs.get('base', None)
             self.klass = attrs['class']
         except KeyError:
-            raise XmlParsingError("'object' items must have a 'class' " \
-                                  "attribute")
+            raise XmlParsingError(_("'object' items must have a 'class' " \
+                                  "attribute"))
         self.parser._objects.push(self)
         self.parent = self.parser._windows.top()
         # -------- added 2002-08-26 to detect container widgets --------------
@@ -689,9 +730,13 @@ class CodeObject:
                 else:
                     self.parser._toplevels.push(self)
             #------------- 2003-05-07: preview --------------------------------
-            elif self.preview and not can_be_toplevel:
+            elif self.preview and not can_be_toplevel and \
+                     self.base != 'CustomWidget':
                 # if this is a custom class, but not a toplevel one,
                 # for the preview we have to use the "real" class
+                #
+                # ALB 2007-08-04: CustomWidgets handle this in a special way
+                # (see widgets/custom_widget/codegen.py)
                 self.klass = self.base
             #------------------------------------------------------------------
             
@@ -729,8 +774,8 @@ class CodeObject:
 ##                     setattr(self.obj, name, flag)
                     self.obj.flag_s = value.strip()
                 else: setattr(self.obj, name, int(value))
-            except: raise XmlParsingError("property '%s' not supported by " \
-                                          "'%s' objects" % (name, self.klass))
+            except: raise XmlParsingError(_("property '%s' not supported by " \
+                                          "'%s' objects") % (name, self.klass))
         self.properties[name] = value
 
     def pop(self):
@@ -776,11 +821,7 @@ class Sizeritem:
                   'wxBOTTOM': wx.BOTTOM,
                   'wxSHAPED': wx.SHAPED,
                   'wxADJUST_MINSIZE': wx.ADJUST_MINSIZE, }
-        import misc
-        if misc.check_wx_version(2, 5, 2):
-            flags['wxFIXED_MINSIZE'] = wx.FIXED_MINSIZE
-        else:
-            flags['wxFIXED_MINSIZE'] = 0
+        flags['wxFIXED_MINSIZE'] = wx.FIXED_MINSIZE
 
     def __init__(self):
         self.option = self.border = 0
